import meteva
import datetime
import pandas as pd
import numpy as np
import copy


def group(sta_ob_and_fos,g = None,gll = None,drop_g_column = False):
    valid_group_list_list = []
    sta_ob_and_fos_list = []
    if g is None:
        sta_ob_and_fos_list.append(sta_ob_and_fos)
    else:
        group_list_list = gll
        if gll is not None:
            group_list_list0 = []
            for group_list in gll:
                if isinstance(group_list,list):
                    group_list_list0.append(group_list)
                else:
                    group_list_list0.append([group_list])
            group_list_list = group_list_list0
        valid_group = ["level","time","time_range","year","month","day","dayofyear","hour","xun",
                       "ob_time","ob_time_range","ob_year","ob_month","ob_day","ob_dayofyear","ob_hour",
                       "dtime","dtime_range","dday","dhour","id","lon","lon_range","lon_step","lat","lat_range","lat_step","last_range","last_step","grid",
                       "province_name","member","ob_day_hour","ob_year_month","day_hour","year_month"]

        data_names = meteva.base.get_stadata_names(sta_ob_and_fos)

        data_names_range = []
        for data_name in data_names:
            data_names_range.append(str(data_name) + "_range")

        data_names_step = []
        for data_name in data_names:
            data_names_step.append(str(data_name) + "_step")

        valid_group.extend(data_names)
        valid_group.extend(data_names_range)
        valid_group.extend(data_names_step)

        if not g in valid_group:
            print("group_by 参数必须为如下列表中的选项：")
            print(str(valid_group))
            return None
        if g in data_names:
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(g)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta = grouped_dict[key]
                    if drop_g_column:
                        sta = sta.drop(labels=g, axis=1)
                    sta_ob_and_fos_list.append(sta)
            else:
                for group_list in group_list_list:
                    sta = meteva.base.in_one_column_value_list(sta_ob_and_fos,g, group_list)
                    if len(sta.index) != 0:
                        valid_group_list_list.append(group_list)
                        if drop_g_column:
                            sta = sta.drop(labels=g, axis=1)
                        sta_ob_and_fos_list.append(sta)

        elif g in data_names_range:
            if group_list_list is None:
                print("当group_by =" + g + "时 group_list_list 参数不能为None")
            else:
                name1 = g.split("_")[0]
                for group_list in group_list_list:
                    sta = meteva.base.between_one_column_value_range(sta_ob_and_fos, name1,group_list[0], group_list[1])
                    if len(sta.index) != 0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)

        elif g in data_names_step:
            if group_list_list is None:
                print("当group_by =" + g + "时 group_list_list 参数不能为None")
            else:
                name1 = g.split("_")[0]
                group_list_list1 = []
                start = group_list_list[0][0]
                step = group_list_list[1][0]
                min = np.min(sta_ob_and_fos.loc[:,name1])
                max = np.max(sta_ob_and_fos.loc[:,name1])
                min =start - (int((start - min) / step) + 1) *step
                max = start + (int((max - start)/step) +1) * step
                value_list = np.arange(min,max,step)
                for value in value_list:
                    group_list_list1.append([value,value+ step - 1e-6])
                for group_list in group_list_list1:
                    sta = meteva.base.between_one_column_value_range(sta_ob_and_fos,name1,group_list[0],group_list[1])
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)

        elif g == "level":
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(g)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = meteva.base.in_level_list(sta_ob_and_fos,group_list)
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)
        elif g == "time":
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(g)))
                keys = grouped_dict.keys()

                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = meteva.base.in_time_list(sta_ob_and_fos,group_list)
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)


        elif g == "time_range":
            if group_list_list is None:
                print("当group_by = time_range时 group_list_list 参数不能为None")
            else:
                for group_list in group_list_list:
                    sta = meteva.base.between_time_range(sta_ob_and_fos,group_list[0],group_list[1])
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)


        elif g == "year":
            fo_times = pd.Series(0, index=sta_ob_and_fos['time'])
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(fo_times.index.year)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = sta_ob_and_fos.loc[fo_times.index.year.isin(group_list)]
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)

        elif g == "month":
            fo_times = pd.Series(0, index=sta_ob_and_fos['time'])
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(fo_times.index.month)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = sta_ob_and_fos.loc[fo_times.index.month.isin(group_list)]
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)
        elif g == "day":
            time0 = datetime.datetime(1900, 1, 1, 0, 0)
            seconds = 3600 * 24
            indexs = (sta_ob_and_fos['time'] - time0) // np.timedelta64(1, "D")
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(indexs)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([time0 + datetime.timedelta(days=key)])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    days_list = []
                    for day0 in group_list:
                        day1 = meteva.base.all_type_time_to_datetime(day0)
                        day = (day1 - time0).total_seconds() // seconds
                        days_list.append(day)
                    sta = sta_ob_and_fos.loc[indexs.isin(days_list)]
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)
        elif g == "dayofyear":
            fo_times = pd.Series(0, index=sta_ob_and_fos['time'])
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(fo_times.index.dayofyear)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = sta_ob_and_fos.loc[fo_times.index.dayofyear.isin(group_list)]
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)
        elif g == "hour":
            fo_times = pd.Series(0, index=sta_ob_and_fos['time'])
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(fo_times.index.hour)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = sta_ob_and_fos.loc[fo_times.index.hour.isin(group_list)]
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)
        elif g == "xun":
            sta_ob_and_fos.reset_index(drop = True,inplace = True)
            fo_times = pd.Series(0, index=sta_ob_and_fos['time'])
            mons = fo_times.index.month.astype(np.int16)
            days = fo_times.index.day.astype(np.int16)
            xuns = np.ceil(days / 10).values.astype(np.int16)
            xuns[xuns > 3] = 3
            xuns += (mons - 1) * 3
            xuns = pd.Series(xuns)
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(xuns)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = sta_ob_and_fos.loc[xuns.isin(group_list)]
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)
        elif g == "day_hour":
            time0 = datetime.datetime(1900, 1, 1, 0, 0)
            seconds = 3600
            indexs = (sta_ob_and_fos['time'] - time0) // np.timedelta64(1, "h")
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(indexs)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([time0 + datetime.timedelta(hours=key)])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    days_list = []
                    for day0 in group_list:
                        day1 = meteva.base.all_type_time_to_datetime(day0)
                        day = (day1 - time0).total_seconds() // seconds
                        days_list.append(day)
                    sta = sta_ob_and_fos.loc[indexs.isin(days_list)]
                    if len(sta.index) != 0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)
        elif g == "year_month":
            sta_ob_and_fos.reset_index(drop = True,inplace = True)
            fo_times = pd.Series(0, index=sta_ob_and_fos['time'])
            year = fo_times.index.year.astype(np.int16)
            month = fo_times.index.month.astype(np.int16)
            yms = year*100+month
            yms = pd.Series(yms)
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(yms)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = sta_ob_and_fos.loc[yms.isin(group_list)]
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)

        elif g.find("ob_") ==0:
            dtime_units = "hour"
            if len(sta_ob_and_fos.attrs) > 0:
                if "dtime_units" in sta_ob_and_fos.attrs:
                    if sta_ob_and_fos.attrs["dtime_units"] != "hour":
                        dtime_units = "minute"
            if dtime_units == "hour":
                dtimes = sta_ob_and_fos["dtime"] * np.timedelta64(1, 'h')
            else:
                dtimes = sta_ob_and_fos["dtime"] * np.timedelta64(1, 'm')

            #dtimes = sta_ob_and_fos["dtime"] * np.timedelta64(1, 'h')
            obtimes = pd.Series(0, index=sta_ob_and_fos['time'] + dtimes)
            if g == "ob_time":
                if group_list_list is None:
                    grouped_dict = dict(list(sta_ob_and_fos.groupby(obtimes.index)))
                    keys = grouped_dict.keys()
                    for key in keys:
                        valid_group_list_list.append([key])
                        sta_ob_and_fos_list.append(grouped_dict[key])
                else:

                    for group_list in group_list_list:
                        group_list1 = []
                        for time_g1 in group_list:
                            group_list1.append(meteva.base.all_type_time_to_datetime(time_g1))
                        sta = sta_ob_and_fos.loc[obtimes.isin(group_list1)]

                        if len(sta.index) !=0:
                            valid_group_list_list.append(group_list1)
                            sta_ob_and_fos_list.append(sta)
            elif g == "ob_time_range":
                if group_list_list is None:
                    print("当group_by = ob_time_range时 group_list_list 参数不能为None")
                else:
                    for group_list in group_list_list:
                        sta = meteva.base.between_ob_time_range(sta_ob_and_fos,group_list[0],group_list[1])
                        if len(sta.index) !=0:
                            valid_group_list_list.append(group_list)
                            sta_ob_and_fos_list.append(sta)

            elif g == "ob_year":
                if group_list_list is None:
                    grouped_dict = dict(list(sta_ob_and_fos.groupby(obtimes.index.year)))
                    keys = grouped_dict.keys()
                    for key in keys:
                        valid_group_list_list.append([key])
                        sta_ob_and_fos_list.append(grouped_dict[key])
                else:
                    for group_list in group_list_list:
                        sta = sta_ob_and_fos.loc[obtimes.index.year.isin(group_list)]
                        if len(sta.index) !=0:
                            valid_group_list_list.append(group_list)
                            sta_ob_and_fos_list.append(sta)

            elif g == "ob_month":
                if group_list_list is None:
                    grouped_dict = dict(list(sta_ob_and_fos.groupby(obtimes.index.month)))
                    keys = grouped_dict.keys()
                    for key in keys:
                        valid_group_list_list.append([key])
                        sta_ob_and_fos_list.append(grouped_dict[key])
                else:
                    for group_list in group_list_list:
                        sta = sta_ob_and_fos.loc[obtimes.index.month.isin(group_list)]
                        if len(sta.index) !=0:
                            valid_group_list_list.append(group_list)
                            sta_ob_and_fos_list.append(sta)
            elif g == "ob_day":
                time0 = datetime.datetime(1900, 1, 1, 0, 0)
                seconds = 3600 * 24
                indexs = (obtimes.index - time0) // np.timedelta64(1, "D")

                if group_list_list is None:
                    grouped_dict = dict(list(sta_ob_and_fos.groupby(indexs)))
                    keys = grouped_dict.keys()
                    for key in keys:
                        valid_group_list_list.append([time0 + datetime.timedelta(days=key)])
                        sta_ob_and_fos_list.append(grouped_dict[key])
                else:
                    for group_list in group_list_list:
                        days_list = []
                        for day0 in group_list:
                            day = (day0 - time0).total_seconds() // seconds
                            days_list.append(day)
                        sta = sta_ob_and_fos.loc[indexs.isin(days_list)]
                        if len(sta.index) !=0:
                            valid_group_list_list.append(group_list)
                            sta_ob_and_fos_list.append(sta)
            elif g == "ob_day_hour":
                time0 = datetime.datetime(1900, 1, 1, 0, 0)
                seconds = 3600
                indexs = (obtimes.index - time0) // np.timedelta64(1, "h")

                if group_list_list is None:
                    grouped_dict = dict(list(sta_ob_and_fos.groupby(indexs)))
                    keys = grouped_dict.keys()
                    for key in keys:
                        valid_group_list_list.append([time0 + datetime.timedelta(hours=key)])
                        sta_ob_and_fos_list.append(grouped_dict[key])
                else:
                    for group_list in group_list_list:
                        days_list = []
                        for day0 in group_list:
                            day = (day0 - time0).total_seconds() // seconds
                            days_list.append(day)
                        sta = sta_ob_and_fos.loc[indexs.isin(days_list)]
                        if len(sta.index) != 0:
                            valid_group_list_list.append(group_list)
                            sta_ob_and_fos_list.append(sta)
            elif g == "ob_dayofyear":
                if group_list_list is None:
                    grouped_dict = dict(list(sta_ob_and_fos.groupby(obtimes.index.dayofyear)))
                    keys = grouped_dict.keys()
                    for key in keys:
                        valid_group_list_list.append([key])
                        sta_ob_and_fos_list.append(grouped_dict[key])
                else:
                    for group_list in group_list_list:
                        sta = sta_ob_and_fos.loc[obtimes.index.dayofyear.isin(group_list)]
                        if len(sta.index) !=0:
                            valid_group_list_list.append(group_list)
                            sta_ob_and_fos_list.append(sta)
            elif g == "ob_hour":
                if group_list_list is None:
                    grouped_dict = dict(list(sta_ob_and_fos.groupby(obtimes.index.hour)))
                    keys = grouped_dict.keys()
                    for key in keys:
                        valid_group_list_list.append([key])
                        sta_ob_and_fos_list.append(grouped_dict[key])
                else:
                    for group_list in group_list_list:
                        sta = sta_ob_and_fos.loc[obtimes.index.hour.isin(group_list)]
                        if len(sta.index) !=0:
                            valid_group_list_list.append(group_list)
                            sta_ob_and_fos_list.append(sta)
            elif g == "ob_xun":
                mons = obtimes.index.month.astype(np.int16)
                days = obtimes.index.day.astype(np.int16)
                xuns = np.ceil(days / 10).values.astype(np.int16)
                xuns[xuns > 3] = 3
                xuns += (mons - 1) * 3
                xuns = pd.Series(xuns)
                if group_list_list is None:
                    grouped_dict = dict(list(sta_ob_and_fos.groupby(xuns)))
                    keys = grouped_dict.keys()
                    for key in keys:
                        valid_group_list_list.append([key])
                        sta_ob_and_fos_list.append(grouped_dict[key])
                else:
                    for group_list in group_list_list:
                        sta = sta_ob_and_fos.loc[xuns.isin(group_list)]
                        if len(sta.index) != 0:
                            valid_group_list_list.append(group_list)
                            sta_ob_and_fos_list.append(sta)

            elif g == "ob_year_month":
                if group_list_list is None:
                    grouped_dict = dict(list(sta_ob_and_fos.groupby(obtimes.index.hour)))
                    keys = grouped_dict.keys()
                    for key in keys:
                        valid_group_list_list.append([key])
                        sta_ob_and_fos_list.append(grouped_dict[key])
                else:
                    for group_list in group_list_list:
                        sta = sta_ob_and_fos.loc[obtimes.index.hour.isin(group_list)]
                        if len(sta.index) != 0:
                            valid_group_list_list.append(group_list)
                            sta_ob_and_fos_list.append(sta)

        elif g == "dtime":
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(g)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = meteva.base.in_dtime_list(sta_ob_and_fos,group_list)
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)

        elif g == "dtime_range":
            if group_list_list is None:
                print("当group_by = dtime_range时 group_list_list 参数不能为None")
            else:
                for group_list in group_list_list:
                    sta = meteva.base.between_dtime_range(sta_ob_and_fos,group_list[0],group_list[1])
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)


        elif g == "dday":
            ddays = np.ceil(sta_ob_and_fos['dtime'] / 24)
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(ddays)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = sta_ob_and_fos.loc[ddays.isin(group_list)]
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)
        elif g == "dhour":
            dhours = sta_ob_and_fos['dtime'] % 24
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(dhours)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = sta_ob_and_fos.loc[dhours.isin(group_list)]
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)
        elif g == "id":
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(g)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = meteva.base.in_id_list(sta_ob_and_fos,group_list)
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)
        elif g =="lon":
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(g)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = meteva.base.in_lon_list(sta_ob_and_fos,group_list)
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)
        elif g =="lat":
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(g)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta_ob_and_fos_list.append(grouped_dict[key])
            else:
                for group_list in group_list_list:
                    sta = meteva.base.in_lat_list(sta_ob_and_fos,group_list)
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)

        elif g == "lon_range":
            if group_list_list is None:
                print("当group_by = lon_range时 group_list_list 参数不能为None")
            else:
                for group_list in group_list_list:
                    sta = meteva.base.between_lon_range(sta_ob_and_fos,group_list[0],group_list[1])
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)

        elif g == "lat_range":
            if group_list_list is None:
                print("当groupy = lat_range时 group_list_list 参数不能为None")
            else:
                for group_list in group_list_list:
                    sta = meteva.base.between_lat_range(sta_ob_and_fos, group_list[0], group_list[1])
                    if len(sta.index) != 0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)

        elif g == "last_range":
            if group_list_list is None:
                print("当group_by = last_range时 group_list_list 参数不能为None")
            else:
                for group_list in group_list_list:
                    sta = meteva.base.between_last_range(sta_ob_and_fos, group_list[0], group_list[1])
                    if len(sta.index) != 0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)

        elif g == "lon_step":
            if group_list_list is None:
                print("当group_by = lon_step时 group_list_list 参数不能为None")
            else:
                group_list_list1 = []
                start = group_list_list[0][0]
                step = group_list_list[1][0]
                min = np.min(sta_ob_and_fos.loc[:,"lon"])
                max = np.max(sta_ob_and_fos.loc[:,"lon"])
                min =start - (int((start - min) / step) + 1) *step
                max = start + (int((max - start)/step) +1) * step
                for value in range(min,max,step):
                    group_list_list1.append([value,value+ step - 1e-6])
                #print(group_list_list1)
                for group_list in group_list_list1:
                    sta = meteva.base.between_lon_range(sta_ob_and_fos,group_list[0],group_list[1])
                    if len(sta.index) !=0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)

        elif g == "lat_step":
            if group_list_list is None:
                print("当group_by = lat_step时 group_list_list 参数不能为None")
            else:
                group_list_list1 = []
                start = group_list_list[0][0]
                step = group_list_list[1][0]
                min = np.min(sta_ob_and_fos.loc[:, "lat"])
                max = np.max(sta_ob_and_fos.loc[:, "lat"])
                min = start - (int((start - min) / step) + 1) * step
                max = start + (int((max - start) / step) + 1) * step
                for value in range(min, max, step):
                    group_list_list1.append([value, value + step - 1e-6])

                for group_list in group_list_list1:
                    sta = meteva.base.between_lat_range(sta_ob_and_fos, group_list[0], group_list[1])
                    if len(sta.index) != 0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)

        elif g == "last_step":
            if group_list_list is None:
                print("当group_by = lat_range时 group_list_list 参数不能为None")
            else:
                group_list_list1 = []
                start = group_list_list[0][0]
                step = group_list_list[1][0]
                min = np.min(sta_ob_and_fos.iloc[:, -1])
                max = np.max(sta_ob_and_fos.iloc[:, -1])

                min = start - (int((start - min) / step) + 1) * step
                max = start + (int((max - start) / step) + 1) * step
                value_list = np.arange(min,max,step)
                for value in value_list:
                    group_list_list1.append([value, value + step - 1e-6])
                for group_list in group_list_list1:
                    sta = meteva.base.between_last_range(sta_ob_and_fos, group_list[0], group_list[1])
                    if len(sta.index) != 0:
                        valid_group_list_list.append(group_list)
                        sta_ob_and_fos_list.append(sta)
        elif g == "grid":
            if group_list_list is None:
                print("当group_by = grid时 group_list_list 参数不能为None")
            for group_list in group_list_list:
                sta = meteva.base.in_grid(sta_ob_and_fos, group_list[0])
                if len(sta.index) != 0:
                    valid_group_list_list.append(group_list)
                    sta_ob_and_fos_list.append(sta)
        elif g == "province_name":
            ids = list(set(sta_ob_and_fos["id"].values))
            sta_province_name = meteva.base.tool.get_station_format_province_set(ids)
            sta_with_province_name = meteva.base.combine_expand_IV(sta_ob_and_fos,sta_province_name)
            grouped_dict = dict(list(sta_with_province_name.groupby(g)))
            keys = grouped_dict.keys()
            for key in keys:
                valid_group_list_list.append([key])
                sta1 = grouped_dict[key].drop([g], axis=1)
                sta_ob_and_fos_list.append(sta1)
        elif g== "member":
            if group_list_list is None:
                group_list_list = meteva.base.get_stadata_names(sta_ob_and_fos)
            for group_list in group_list_list:
                sta = meteva.base.in_member_list(sta_ob_and_fos,group_list)
                if sta is not None and len(sta.index) != 0:
                    valid_group_list_list.append(group_list)
                    sta_ob_and_fos_list.append(sta)

        else:
            if group_list_list is None:
                grouped_dict = dict(list(sta_ob_and_fos.groupby(g)))
                keys = grouped_dict.keys()
                for key in keys:
                    valid_group_list_list.append([key])
                    sta1 = grouped_dict[key].drop([g],axis = 1)
                    #print(sta1)
                    sta_ob_and_fos_list.append(sta1)
            else:
                pass
                #for group_list in group_list_list:
                #    sta = meteva.base.in_one_column_list(sta_ob_and_fos, group_list)
                #    if len(sta.index) != 0:
                #        valid_group_list_list.append(group_list)
                #        sta_ob_and_fos_list.append(sta)


    #返回分组结果，和实际分组方式
    if len(valid_group_list_list)==0:
        valid_group_list = None
    else:

        try:
            vg =np.array(valid_group_list_list)
            if len(valid_group_list_list) > 1:
                valid_group_list =  vg.squeeze().tolist()
            else:
                if vg.size == 1:
                    valid_group_list = valid_group_list_list[0]
                else:
                    valid_group_list = valid_group_list_list
        except:
            valid_group_list = valid_group_list_list

    return sta_ob_and_fos_list,valid_group_list


def split(sta_ob_and_fos,used_coords = ["level","time","dtime"],sta_list = None):
    '''

    :param sta_ob_and_fos: 包含多个层次，时间，时效，站点的观测和预报数据
    :param used_coords: 拆分的维度
    :param sta_list: 最终返回的结果
    :return:
    '''

    if sta_list is None:
        sta_list = []
    sta_group = group(sta_ob_and_fos, g=used_coords[0])[0]
    if len(used_coords) >1:
        # 取出第一个coord
        for sta in sta_group:
            split(sta,used_coords=used_coords[1:],sta_list = sta_list)
    else:
        for sta in sta_group:
            sta_list.append(sta)

    return sta_list

def split_grd(grd,used_coords = ["member","level","time","dtime"],grd_list = None):
    '''

    :param sta_ob_and_fos: 包含多个层次，时间，时效，站点的观测和预报数据
    :param used_coords: 拆分的维度
    :param sta_list: 最终返回的结果
    :return:
    '''

    if grd_list is None:
        grd_list = []
    grd_group = group_grd(grd, g=used_coords[0])
    if len(used_coords) >1:
        # 取出第一个coord
        for grd in grd_group:
            split_grd(grd,used_coords=used_coords[1:],grd_list = grd_list)
    else:
        for grd in grd_group:
            grd_list.append(grd)
    return grd_list


def group_grd(grd, g = None):
    if g == None:
        return [grd]
    else:
        ng = len(grd[g].values)
        grd_list = []
        for i in range(ng):
            if g =="level":
                grd1 = grd.isel(level = slice(i,i+1))
            elif g == "time":
                grd1 = grd.isel(time=slice(i, i+1))
            elif g == "dtime":
                grd1 = grd.isel(dtime=slice(i, i+1))
            elif g == "member":
                grd1 = grd.isel(member=slice(i, i+1))
            grd_list.append(grd1)
        return grd_list


import copy
import numpy as np
from .utils import get_attributes_for_feat, remove_key_from_list
from .feature_props import feature_props
from .feature_comps import feature_comps
from .feature_table import feature_table
from .interester import interester
from .feature_axis import feature_axis
import meteva


def feature_match_analyzer(look_match, which_comps=None, sizefac=1, alpha=0.1, k=1, p=2,
                           c=float("inf"), distfun="distmapfun", y=None, matches=None, object=None):


    x = copy.deepcopy(look_match)
    if which_comps is None:
        which_comps = ["cent.dist", "angle.diff", "area.ratio",
                       "int.area", "bdelta", "haus", "ph", "med", "msd", "fom",
                       "minsep", "bearing"]
    if x['match_type'] == "centmatch" or x['match_type'] == "minboundmatch" or x["match_type"] == "deltamm" or x["match_type"][0] == "MergeForce":
        n = x['matches'].shape[0]
        if n > 0:
            out = []
            a = x
            ## loc 没有值
            if "loc" in a.keys():
                loc = a['loc']
            else:
                loc = None
            #Xfeats = x['Xlabelsfeature']
            #Yfeats = x['Ylabelsfeature']
            Xfeats = x['grd_ob_features']
            Yfeats = x['grd_fo_features']
            #xattribute = get_attributes_for_feat(Xfeats)
            #yattribute = get_attributes_for_feat(Yfeats)
            remove_list = ['Type', 'xrange', 'yrange', 'dim', 'xstep', 'ystep', 'warnings', 'xcol', 'yrow', 'area','label_count']
            #print(Xfeats.keys())
            xkeys = remove_key_from_list(list(Xfeats.keys()), remove_list)
            ykeys = remove_key_from_list(list(Yfeats.keys()), remove_list)
            for i in range(n):
                j = x['matches'][i, 0]-1
                k = x['matches'][i, 1]-1
                ymat = Yfeats[ykeys[j]]
                xmat = Xfeats[xkeys[k]]

                #if xmat.dtype.name is not 'bool':
                #    xmat = (xmat == 1)
                #if ymat.dtype.name is not 'bool':
                #    ymat = (ymat == 1)
                #ymat = {"m": ymat}
                #ymat.update(yattribute)
                #xmat = {"m": xmat}
                #xmat.update(xattribute)
                #out.append(feature_comps(grd_fo=ymat, grd_ob=xmat, which_comps=which_comps,
                #                         sizefac=sizefac, alpha=alpha, k=k, p=p, c=c, distfun=distfun, loc=loc))
                out.append(feature_comps(look_match,label_ob=k+1,label_fo=j+1, which_comps=which_comps,
                                         sizefac=sizefac, alpha=alpha, k=k, p=p, c=c, distfun=distfun, loc=loc))
        else:
            out = "No matches found"
        return out

    elif x['match_type'] == "deltamm":
        if matches is not None:
            obj = matches
        elif y is not None:
            obj = y
        else:
            obj = x
        if "loc" in obj.keys():
            loc = obj['loc']
        else:
            loc = None
        Yfeats = obj['Ylabelsfeature']
        Xfeats = obj['Xlabelsfeature']
        xattribute = get_attributes_for_feat(Xfeats)
        yattribute = get_attributes_for_feat(Yfeats)
        if obj['unmatched']['matches'].shape[0] == 0:
            out = "No matches found"
        else:
            n = obj['unmatched']['matches'].shape[0]
            out = []
            remove_list = ['Type', 'xrange', 'yrange', 'dim', 'xstep', 'ystep', 'warnings', 'xcol', 'yrow', 'area']
            xkeys = remove_key_from_list(list(Xfeats.keys()), remove_list)
            ykeys = remove_key_from_list(list(Yfeats.keys()), remove_list)
            for i in range(n):
                ymat = Yfeats[ykeys[i]]
                xmat = Xfeats[xkeys[i]]
                if xmat.dtype.name is not 'bool':
                    xmat = (xmat == 1)
                if ymat.dtype.name is not 'bool':
                    ymat = (ymat == 1)
                ymat = {"m": ymat}
                ymat.update(yattribute)
                xmat = {"m": xmat}
                xmat.update(xattribute)
                out.append(feature_comps(Y=ymat, X=xmat, which_comps=which_comps, sizefac=sizefac,
                                         alpha=alpha, k=k, p=p, c=c, distfun=distfun, loc=loc))
        return out
    else:
        print("类型错误")
        raise Exception("类型错误")

'''
if __name__ == '__main__':
    data1 = np.load("../../centmatchResult.npy", allow_pickle=True).tolist()
    data2 = np.load("../../deltammResult_PA2.npy", allow_pickle=True).tolist()
    data1['Xlabelsfeature'] = data2['Xlabelsfeature']
    data1['Ylabelsfeature'] = data2['Ylabelsfeature']
    out1 = feature_match_analyzer(data1)
    out2 = feature_match_analyzer(data2)
    print("hello")
'''


def get_summary(feature):
    out = copy.deepcopy(feature)
    #print(out.keys())
    if "is_summary" not in out.keys():
        out["is_summary"] = True
        if out["interester"] is not None:
            nmatch = out["match_count"]
            interest = out["interester"]
            index = np.arange(nmatch)
            dat = interest["total_interest"][index, index]
            out["interester"] = dat

        label_list_matched = out["label_list_matched"]
        for i in label_list_matched:
            del out[i]["feature_axis"]["ob"]["point"]
            del out[i]["feature_axis"]["ob"]["MajorAxis"]
            del out[i]["feature_axis"]["ob"]["MinorAxis"]
            del out[i]["feature_axis"]["ob"]["aspect_ratio"]
            del out[i]["feature_axis"]["ob"]["OrientationAngle"]["MinorAxis"]
            del out[i]["feature_axis"]["ob"]["pts"]
            del out[i]["feature_axis"]["ob"]["MidPoint"]
            del out[i]["feature_axis"]["ob"]["sma_fit"]
            del out[i]["feature_axis"]["ob"]["phi"]

            del out[i]["feature_axis"]["fo"]["point"]
            del out[i]["feature_axis"]["fo"]["MajorAxis"]
            del out[i]["feature_axis"]["fo"]["MinorAxis"]
            del out[i]["feature_axis"]["fo"]["aspect_ratio"]
            del out[i]["feature_axis"]["fo"]["OrientationAngle"]["MinorAxis"]
            del out[i]["feature_axis"]["fo"]["pts"]
            del out[i]["feature_axis"]["fo"]["MidPoint"]
            del out[i]["feature_axis"]["fo"]["sma_fit"]
            del out[i]["feature_axis"]["fo"]["phi"]

            del out[i]["feature_props"]["ob"]["axis"]
            del out[i]["feature_props"]["fo"]["axis"]

        miss_labels = out["unmatched"]['ob']

        for i in miss_labels:
            del out[i]["feature_axis"]["ob"]["point"]
            del out[i]["feature_axis"]["ob"]["MajorAxis"]
            del out[i]["feature_axis"]["ob"]["MinorAxis"]
            del out[i]["feature_axis"]["ob"]["aspect_ratio"]
            del out[i]["feature_axis"]["ob"]["OrientationAngle"]["MinorAxis"]
            del out[i]["feature_axis"]["ob"]["pts"]
            del out[i]["feature_axis"]["ob"]["MidPoint"]
            del out[i]["feature_axis"]["ob"]["sma_fit"]
            del out[i]["feature_axis"]["ob"]["phi"]
            del out[i]["feature_props"]["ob"]["axis"]


        false_alarm_labels = out["unmatched"]["fo"]
        for i in false_alarm_labels:
            del out[i]["feature_axis"]["fo"]["point"]
            del out[i]["feature_axis"]["fo"]["MajorAxis"]
            del out[i]["feature_axis"]["fo"]["MinorAxis"]
            del out[i]["feature_axis"]["fo"]["aspect_ratio"]
            del out[i]["feature_axis"]["fo"]["OrientationAngle"]["MinorAxis"]
            del out[i]["feature_axis"]["fo"]["pts"]
            del out[i]["feature_axis"]["fo"]["MidPoint"]
            del out[i]["feature_axis"]["fo"]["sma_fit"]
            del out[i]["feature_axis"]["fo"]["phi"]
            del out[i]["feature_props"]["fo"]["axis"]

    return out

def feature_merged_analyzer(look_merge,summary =True):
    nmatch = look_merge["match_count"]
    out = {}
    out["time"] = meteva.base.all_type_time_to_str(look_merge["grd_fo"]["time"].values[0])
    out["dtime"] = int(look_merge["grd_fo"]["dtime"].values[0])
    out["member"] = look_merge["grd_fo"]["member"].values[0]
    out["match_count"] = nmatch
    f_table = feature_table(look_merge)
    out["feature_table"] = f_table
    interest = interester(look_merge)
    out["interester"] = interest

    label_list_matched = look_merge["label_list_matched"]
    out["label_list_matched"] = label_list_matched
    out["unmatched"] = look_merge["unmatched"]

    for i in label_list_matched:
        f_axis_ob = feature_axis(look_merge, i, "ob")
        f_axis_fo = feature_axis(look_merge, i, "fo")
        f_props_ob = feature_props(look_merge, i, "ob")
        f_props_fo = feature_props(look_merge, i, "fo")
        f_comps = feature_comps(look_merge, i, i)
        out1 = {}
        #print(f_axis_ob.keys())

        out1["feature_axis"] = {"ob":f_axis_ob,"fo":f_axis_fo}
        out1["feature_props"]= {"ob":f_props_ob,"fo":f_props_fo}
        out1["feature_comps"] = f_comps

        out[i] = out1

    miss_labels = look_merge["unmatched"]['ob']
    for i in miss_labels:
        f_axis_ob = feature_axis(look_merge, i, "ob")
        f_props_ob = feature_props(look_merge, i, "ob")
        out1 = {}
        out1["feature_axis"] = {"ob":f_axis_ob}
        out1["feature_props"]= {"ob":f_props_ob}
        out[i] = out1


    false_alarm_labels = look_merge["unmatched"]["fo"]
    for i in false_alarm_labels:
        f_axis_fo = feature_axis(look_merge, i, "fo")
        f_props_fo = feature_props(look_merge, i, "fo")
        if i in out.keys():
            out1 = out[i]
            out1["feature_axis"]["fo"]= f_axis_fo
            out1["feature_props"]["fo"]= f_props_fo
        else:
            out1 = {}
            out1["feature_axis"] = {"fo": f_axis_fo}
            out1["feature_props"] = {"fo": f_props_fo}

        out[i] = out1

    if summary:
        out = get_summary(out)

    return out


import meteva
import numpy as np
import datetime
import copy
import time
import pandas as pd
import os
import math

method_coluns_dict = {
    "hfmc_of_sun_rain":["Hsr","Fsr","Msr","Csr"],
    "hfmc" :  ["H","F","M","C"],
    "hfmc_multi": ["H", "F", "M", "C"],
    "tase":["T", "E", "A", "S"],
    "tc_count":["T", "C"],
    "tlfo":["T","LFO"],
    "toar":["T","OAR"],
    "tmmsss": ["T", "MX","MY","SX","SY","SXY"],
    "tbask":["T","BASK"],
    "nasws_uv": ["T", "AC", "SCORE", "SEVERER", "WEAK"],
    "nas_uv": ["T", "AC", "SCORE"],
    "na_uv":["T","AC"],
    "nasws_s": ["T", "AC", "SCORE", "SEVERER", "WEAK"],
    "fss":["pob","pfo","fbs"],
    "tems":["T","E","MX","SX"],
    "cscs":["CX","SX","CY","SY"],
    "hnh": np.arange(20).tolist()

}

def get_score_method_with_mid(method):
    hfmc_list = [
        meteva.method.ts,meteva.method.bias,meteva.method.ets,meteva.method.mr,meteva.method.far,
        meteva.method.r,meteva.method.hss_yesorno,meteva.method.pofd,meteva.method.pod,
        meteva.method.dts,meteva.method.orss,meteva.method.pc,meteva.method.roc,meteva.method.sr,
        meteva.method.hk_yesorno,meteva.method.odds_ratio,meteva.method.ob_fo_hr,meteva.method.ob_fo_hc,
        meteva.method.false_alarm_count
    ]
    hfmc_multi_list = [
        meteva.method.ts_multi,meteva.method.bias_multi,meteva.method.ets_multi,meteva.method.mr_multi,meteva.method.far_multi,
        meteva.method.pofd_multi,meteva.method.pod_multi,meteva.method.sr_multi]

    hfmc_of_sum_rain_list = [meteva.method.pc_of_sun_rain]
    tase_list = [meteva.method.me, meteva.method.mae, meteva.method.mse, meteva.method.rmse]
    tc_list = [meteva.method.correct_rate,meteva.method.wrong_rate]
    tmmsss_list = [meteva.method.residual_error, meteva.method.residual_error_rate,meteva.method.corr,
                    meteva.method.nse,meteva.method.ob_fo_mean,meteva.method.ob_fo_sum,meteva.method.ob_fo_std]
    tbask_list = [meteva.product.regulation.temperature.temp_forecaster_score]
    nasws_s_list = [meteva.method.acs, meteva.method.scs, meteva.method.wind_weaker_rate,
                     meteva.method.wind_severer_rate]
    nasws_uv_list = [meteva.method.acs_uv,meteva.method.scs_uv,meteva.method.wind_weaker_rate_uv,meteva.method.wind_severer_rate_uv]
    nas_uv_list = [meteva.method.acd_uv,meteva.method.scd_uv]
    na_uv_list = [meteva.method.acz_uv]
    toar_list = [meteva.method.mre]
    tlfo_list = [meteva.method.rmsf]
    fss_list = [meteva.method.fss]
    tems_list = [meteva.method.bs,meteva.method.bss]
    hnh_list = [meteva.method.roc_auc]
    cscs_list = [meteva.method.ob_fo_precipitation_strength]

    method_mid = None
    method_name = method.__name__

    if method in hfmc_of_sum_rain_list:
        method_mid = getattr(meteva.method, method_name + "_hfmc")
    elif method in hfmc_list:
        method_mid  = getattr(meteva.method, method_name +"_hfmc")
    elif method in hfmc_multi_list:
        method_mid  = getattr(meteva.method, method_name[:-6] +"_hfmc")
    elif method in tase_list:
        method_mid = getattr(meteva.method, method_name + "_tase")
    elif method in tc_list:
        method_mid = getattr(meteva.method, method_name + "_tc")
    elif method in tmmsss_list:
        method_mid = getattr(meteva.method, method_name + "_tmmsss")
    elif method in toar_list:
        method_mid = getattr(meteva.method, method_name + "_toar")
    elif method in tlfo_list:
        method_mid = getattr(meteva.method, method_name + "_tlfo")
    elif method in tbask_list:
        method_mid = getattr(meteva.product, method_name + "_tbask")
    elif method in nasws_s_list:
        method_mid = getattr(meteva.method, method_name + "_nasws")
    elif method in tems_list:
        method_mid = getattr(meteva.method, method_name + "_tems")
    elif method in nasws_uv_list:
        method_name1 = method_name.replace("_uv","_nasws")
        method_mid = getattr(meteva.method, method_name1)
    elif method in nas_uv_list:
        method_name1 = method_name.replace("_uv","_nas")
        method_mid = getattr(meteva.method, method_name1)
    elif method in na_uv_list:
        method_name1 = method_name.replace("_uv","_na")
        method_mid = getattr(meteva.method, method_name1)
    elif method in fss_list:
        method_mid = getattr(meteva.method, method_name + "_fof")
    elif method in hnh_list:
        method_mid = getattr(meteva.method, method_name + "_hnh")
    elif method in cscs_list:
        method_mid = getattr(meteva.method, method_name + "_cscs")
    #print(method_mid)
    return method_mid

def get_middle_method(method):
    hfmc_list = [
        meteva.method.ts,meteva.method.bias,meteva.method.ets,meteva.method.mr,meteva.method.far,
        meteva.method.r,meteva.method.hss_yesorno,meteva.method.pofd,meteva.method.pod,
        meteva.method.dts,meteva.method.orss,meteva.method.pc,meteva.method.roc,meteva.method.sr,
        meteva.method.hk_yesorno,meteva.method.odds_ratio,meteva.method.ob_fo_hr,meteva.method.ob_fo_hc,
        meteva.method.false_alarm_count
    ]
    hfmc_multi_list = [
        meteva.method.ts_multi,meteva.method.bias_multi,meteva.method.ets_multi,meteva.method.mr_multi,meteva.method.far_multi,
        meteva.method.pofd_multi,meteva.method.pod_multi,meteva.method.sr_multi]

    hfmc_of_sum_rain_list = [meteva.method.pc_of_sun_rain]
    tase_list = [meteva.method.me, meteva.method.mae, meteva.method.mse, meteva.method.rmse]
    tc_list = [meteva.method.correct_rate,meteva.method.wrong_rate]
    tmmsss_list = [meteva.method.residual_error, meteva.method.residual_error_rate,meteva.method.corr,meteva.method.ob_fo_sum,meteva.method.ob_fo_mean,
                   meteva.method.nse,meteva.method.ob_fo_std]
    toar_list = [meteva.method.mre]
    tlfo_list = [meteva.method.rmsf]
    tbask_list = [meteva.product.regulation.temperature.temp_forecaster_score]
    nasws_s_list = [meteva.method.acs,meteva.method.scs,meteva.method.wind_weaker_rate,meteva.method.wind_severer_rate]
    nasws_uv_list = [meteva.method.acs_uv,meteva.method.scs_uv,meteva.method.wind_weaker_rate_uv,meteva.method.wind_severer_rate_uv]
    nas_uv_list = [meteva.method.acd_uv,meteva.method.scd_uv]
    na_uv_list = [meteva.method.acz_uv]
    fss_list = [meteva.method.fss]
    tems_list = [meteva.method.bs, meteva.method.bss]
    hnh_list = [meteva.method.roc_auc]
    cscs_list = [meteva.method.ob_fo_precipitation_strength]
    method_mid = None
    if method in hfmc_of_sum_rain_list:
        method_mid = meteva.method.hfmc_of_sun_rain
    elif method in hfmc_list:
        method_mid  = meteva.method.hfmc
    elif method in hfmc_multi_list:
        method_mid = meteva.method.hfmc_multi
    elif method in tase_list:
        method_mid = meteva.method.tase
    elif method in tc_list:
        method_mid = meteva.method.tc_count
    elif method in tmmsss_list:
        method_mid = meteva.method.tmmsss
    elif method in toar_list:
        method_mid = meteva.method.toar
    elif method in tlfo_list:
        method_mid = meteva.method.tlfo
    elif method in tbask_list:
        method_mid = meteva.product.regulation.temperature.tbask
    elif method in nasws_s_list:
        method_mid = meteva.method.nasws_s
    elif method in nasws_uv_list:
        method_mid = meteva.method.nasws_uv
    elif method in nas_uv_list:
        method_mid = meteva.method.nas_uv
    elif method in na_uv_list:
        method_mid = meteva.method.na_uv
    elif method in fss_list:
        method_mid = meteva.method.fss
    elif method in tems_list:
        method_mid = meteva.method.tems
    elif method in hnh_list:
        method_mid = meteva.method.hnh
    elif method in cscs_list:
        method_mid = meteva.method.cscs
    return method_mid

def get_middle_columns(method):

    method_name = method.__name__
    if method_name in method_coluns_dict.keys():
        return method_coluns_dict[method_name]
    else:
        print("暂无支持")
        pass

def middle_df_sta_bak2(sta_all,method,grade_list = None,compare = None,gid = None):
    '''

    :param sta_all:
    :param method:
    :param grade_list:
    :param compare:
    :param gid:
    :return:
    '''

    need_g = False
    if gid is not None:
        need_g = True

    mid_columns = get_middle_columns(method)
    method_args = {}
    if grade_list is not None:
        method_args["grade_list"] = grade_list
    if compare is not None:
        method_args["compare"] = compare
    grade_exp = None
    names_exp = None
    group_name = None
    if need_g:
        group_name = gid.columns[1]
        groups = gid[group_name]
        groups = groups.drop_duplicates(keep = "first")
        names = groups.values
        gll = []
        for i in range(len(names)):
            ids = gid.loc[gid[group_name] == names[i]].values[:, 0]
            gll.append(ids.tolist())

        method_args["g"] = "id"
        method_args["gll"] = gll
        if grade_list is not None:
            grade_exp, names_exp = np.meshgrid(grade_list, names)
            grade_exp = grade_exp.flatten().tolist()
            names_exp = names_exp.flatten().tolist()
        else:
            names_exp = names

        sta_all_gid = meteva.base.combine_expand(sta_all,gid)
    else:
        sta_all_gid = sta_all
        if grade_list is not None:
            grade_exp = grade_list
        else:
            pass


    sta_list = meteva.base.split(sta_all_gid,used_coords=["level","time","dtime",group_name])
    print(sta_list[0])
    df_list = []
    for k in range(len(sta_list)):
        sta = sta_list[k]


        not_all_iv = [True]
        if method.__name__.find("_uv") >= 0 or method.__name__.find("distance") >= 0:
            col_step = 2
            valid_index = [0, 1]
        else:
            col_step = 1
            valid_index = [0]

        len_ = len(sta.columns)
        for nv in range(6 + col_step, len_, col_step):
            not_all_iv1 = np.any(sta.iloc[:, nv].values != meteva.base.IV)
            not_all_iv.append(not_all_iv1)
            if not_all_iv1:
                valid_index.append(nv - 6)
                if col_step == 2: valid_index.append(nv - 5)

        sta = meteva.base.in_member_list(sta, member_list=valid_index, name_or_index="index")
        sta1 = meteva.base.not_IV(sta)
        print(sta1)
        if len(sta1.index)==0:continue
        data_names = meteva.base.get_stadata_names(sta1)
        if method.__name__=="tbask":
            mid_array, _ = meteva.product.score(sta1, method)
            dict1 = {"time": sta["time"].values[0], "dtime": sta["dtime"].values[0],"T":mid_array[0],"BASK":mid_array[1]}
            tbask_df = pd.DataFrame(dict1,index = [k])
            df_list.append(tbask_df)
        else:

            for m in range(1, len(data_names)):
                sta2 = meteva.base.sele_by_para(sta1, member=[data_names[0], data_names[m]])

                mid_array, gll_valid = meteva.product.score(sta2, method,**method_args,drop_g_column=True)

                index_names = ["time","dtime","member"]
                dict1 = {"time":sta2["time"].values[0],"dtime":  sta2["dtime"].values[0],"member": data_names[m]}
                if names_exp is not None:
                    index_names.append(group_name)
                    dict1[group_name] = names_exp
                if grade_exp is not None:
                    index_names.append("grade")
                    dict1["grade"] = grade_exp

                if method == meteva.method.tc_count:
                    if grade_list is None:
                        dat1 = mid_array[..., 0]
                    else:
                        dat1 = np.repeat([mid_array[..., 0]],len(grade_list))
                    dict1[mid_columns[0]] = dat1.flatten()
                    dict1[mid_columns[1]] = mid_array[..., 1:].flatten()
                else:
                    for c in range(len(mid_columns)):
                        dict1[mid_columns[c]] = mid_array[...,c].flatten()


                hfmc_df = pd.DataFrame(dict1)
                df_list.append(hfmc_df)
    df_all = pd.concat(df_list, axis=0)
    return df_all

def middle_df_sta(sta_all,method,grade_list = None,compare = None,gid = None):
    '''

    :param sta_all:
    :param method:
    :param grade_list:
    :param compare:
    :param gid:
    :return:
    '''

    need_g = False
    if gid is not None:
        need_g = True

    mid_columns = get_middle_columns(method)
    method_args = {}
    if grade_list is not None:
        method_args["grade_list"] = grade_list
    if compare is not None:
        method_args["compare"] = compare
    grade_exp = None
    names_exp = None
    group_name = None
    if need_g:
        #may be some id in gid is not in sta_all,valid group is not same as group in grid
        sta_all_id = copy.deepcopy(sta_all[["id"]])
        sta_all_id.drop_duplicates()
        sta = meteva.base.combine_on_id(gid,sta_all_id,how="inner")
        gid = sta
        group_name = gid.columns[1]
        groups = gid[group_name]
        groups = groups.drop_duplicates(keep = "first")
        names = groups.values
        gll = []
        for i in range(len(names)):
            ids = gid.loc[gid[group_name] == names[i]].values[:, 0]
            gll.append(ids.tolist())

        method_args["g"] = "id"
        method_args["gll"] = gll
        if grade_list is not None:
            if method == meteva.method.hfmc_multi:
                grade_list_name = ["<" + str(grade_list[0])]
                ng = len(grade_list)
                for kk in range(ng - 1):
                    grade_list_name.append("[" + str(grade_list[kk]) + "," + str(grade_list[kk + 1]) + ")")
                grade_list_name.append(">" + str(grade_list[ng - 1]))
                grade_exp, names_exp = np.meshgrid(grade_list, names)
            else:
                grade_exp, names_exp = np.meshgrid(grade_list, names)
            grade_exp = grade_exp.flatten().tolist()
            names_exp = names_exp.flatten().tolist()
        else:
            names_exp = names

        #sta_all_gid = meteva.base.combine_expand(sta_all,gid)
    else:
        #sta_all_gid = sta_all
        if grade_list is not None:
            if method == meteva.method.hfmc_multi:
                grade_list_name = ["<" + str(grade_list[0])]
                ng = len(grade_list)
                for kk in range(ng- 1):
                    grade_list_name.append("[" + str(grade_list[kk]) + "," + str(grade_list[kk + 1]) + ")")
                grade_list_name.append(">" + str(grade_list[ng-1]))
                grade_exp = grade_list_name
            else:
                grade_exp = grade_list
        else:
            pass

    sta_list = meteva.base.split(sta_all,used_coords=["level","time","dtime"])
    df_list = []
    for k in range(len(sta_list)):
        sta = sta_list[k]

        not_all_iv = [True]
        if method.__name__.find("_uv") >= 0 or method.__name__.find("distance") >= 0:
            col_step = 2
            valid_index = [0, 1]
        else:
            col_step = 1
            valid_index = [0]

        len_ = len(sta.columns)
        for nv in range(6 + col_step, len_, col_step):
            not_all_iv1 = np.any(sta.iloc[:, nv].values != meteva.base.IV)
            not_all_iv.append(not_all_iv1)
            if not_all_iv1:
                valid_index.append(nv - 6)
                if col_step == 2: valid_index.append(nv - 5)

        sta = meteva.base.in_member_list(sta, member_list=valid_index, name_or_index="index")
        sta1 = meteva.base.not_IV(sta)
        if len(sta1.index)==0:continue
        data_names = meteva.base.get_stadata_names(sta1)
        member_count =int(len(data_names)/col_step)
        if method.__name__=="tbask":
            mid_array, _ = meteva.product.score(sta1, method)
            dict1 = {"time": sta["time"].values[0], "dtime": sta["dtime"].values[0],"T":mid_array[0],"BASK":mid_array[1]}
            tbask_df = pd.DataFrame(dict1,index = [k])
            df_list.append(tbask_df)
        elif method.__name__ =="hnh":
            for m in range(1, member_count):
                sta2 = meteva.base.sele_by_para(sta1, member=[data_names[0], data_names[m]])
                data_name  = data_names[m]
                mid_array, _ = meteva.product.score(sta2, method)
                mid_array1 = mid_array.flatten()
                #print(mid_array)
                dict1 = {"time": sta["time"].values[0], "dtime": sta["dtime"].values[0],"member":data_name}
                for i in range(20):
                    dict1[i] = mid_array1[i]
                hnh_df = pd.DataFrame(dict1, index=[k])
                #print(hnh_df)
                df_list.append(hnh_df)

        else:
            for m in range(1, member_count):
                if col_step==1:
                    sta2 = meteva.base.sele_by_para(sta1, member=[data_names[0], data_names[m]])
                    data_name  = data_names[m]
                else:
                    sta2 = meteva.base.sele_by_para(sta1, member=[data_names[0],data_names[1], data_names[m*2],data_names[m*2+1]])
                    data_name = data_names[m*2]
                    data_name = data_name[data_name.find("_")+1:]
                mid_array, gll_valid = meteva.product.score(sta2, method,**method_args,drop_g_column=True)
                if mid_array is None: continue

                index_names = ["time","dtime","member"]
                dict1 = {"time":sta2["time"].values[0],"dtime":  sta2["dtime"].values[0],"member":data_name}
                if names_exp is not None:
                    index_names.append(group_name)
                    dict1[group_name] = names_exp
                if grade_exp is not None:
                    index_names.append("grade")
                    dict1["grade"] = grade_exp

                if method == meteva.method.tc_count:
                    if grade_list is None:
                        dat1 = mid_array[..., 0]
                    else:
                        dat1 = np.repeat([mid_array[..., 0]],len(grade_list))
                    dict1[mid_columns[0]] = dat1.flatten()
                    dict1[mid_columns[1]] = mid_array[..., 1:].flatten()
                else:
                    for c in range(len(mid_columns)):
                        dict1[mid_columns[c]] = mid_array[...,c].flatten()
                try:
                    hfmc_df = pd.DataFrame(dict1)
                    df_list.append(hfmc_df)
                except:
                    ob_time = meteva.base.all_type_time_to_datetime(sta2["time"].values[0]) + datetime.timedelta(hours=int(sta2["dtime"].values[0]))
                    time_str = meteva.base.all_type_time_to_str(sta2["time"].values[0])
                    time_str_ob = meteva.base.all_type_time_to_str(ob_time)
                    print(data_names[m] + "的" + time_str+"起报的"+str(sta2["dtime"].values[0])+"时效预报的空间范围不完整,对应的实况时间为："+time_str_ob)
    df_all = pd.concat(df_list, axis=0)
    return df_all


def middle_df_sta_bak1(sta_all,method,grade_list = None,compare = None,gid = None):
    '''

    :param sta_all:
    :param method:
    :param grade_list:
    :param compare:
    :param gid:
    :return:
    '''

    need_g = False
    if gid is not None:
        need_g = True


    mid_columns = get_middle_columns(method)
    method_args = {}
    if grade_list is not None:
        method_args["grade_list"] = grade_list
    if compare is not None:
        method_args["compare"] = compare


    if need_g:
        group_name = gid.columns[1]
        sta_all_gid = meteva.base.combine_expand(sta_all,gid)
        sta_list = meteva.base.split(sta_all_gid, used_coords=["level", "time", "dtime", group_name])
    else:
        sta_list = meteva.base.split(sta_all,used_coords=["level","time","dtime"])


    df_list = []
    for k in range(len(sta_list)):
        print(str(k)+"/"+str(len(sta_list)))
        sta = sta_list[k]
        if need_g:
            group_value = sta.iloc[0,-1]
            sta = sta.iloc[:,:-1]

        not_all_iv = [True]
        if method.__name__.find("_uv") >= 0 or method.__name__.find("distance") >= 0:
            col_step = 2
            valid_index = [0, 1]
        else:
            col_step = 1
            valid_index = [0]

        len_ = len(sta.columns)
        for nv in range(6 + col_step, len_, col_step):
            not_all_iv1 = np.any(sta.iloc[:, nv].values != meteva.base.IV)
            not_all_iv.append(not_all_iv1)
            if not_all_iv1:
                valid_index.append(nv - 6)
                if col_step == 2: valid_index.append(nv - 5)

        sta = meteva.base.in_member_list(sta, member_list=valid_index, name_or_index="index")
        sta1 = meteva.base.not_IV(sta)
        #print(sta1)
        if len(sta1.index)==0:continue
        data_names = meteva.base.get_stadata_names(sta1)
        if method.__name__=="tbask":
            mid_array, _ = meteva.product.score(sta1, method)
            dict1 = {"time": sta["time"].values[0], "dtime": sta["dtime"].values[0],"T":mid_array[0],"BASK":mid_array[1]}
            tbask_df = pd.DataFrame(dict1,index = [k])
            df_list.append(tbask_df)
        else:

            for m in range(1, len(data_names)):
                sta2 = meteva.base.sele_by_para(sta1, member=[data_names[0], data_names[m]])

                mid_array, gll1 = meteva.product.score(sta2, method,**method_args)

                index_names = ["time","dtime","member"]
                dict1 = {"time":sta2["time"].values[0],"dtime":  sta2["dtime"].values[0],"member": data_names[m]}

                if grade_list is not None:
                    index_names.append("grade")
                    dict1["grade"] = grade_list
                if need_g:
                    dict1[group_name] = group_value

                #print(dict1)
                if method == meteva.method.tc_count:
                    if grade_list is None:
                        dat1 = mid_array[..., 0]
                    else:
                        dat1 = np.repeat([mid_array[..., 0]],len(grade_list))
                    dict1[mid_columns[0]] = dat1.flatten()
                    dict1[mid_columns[1]] = mid_array[..., 1:].flatten()
                else:
                    for c in range(len(mid_columns)):
                        dict1[mid_columns[c]] = mid_array[...,c].flatten()
                        pass
                hfmc_df = pd.DataFrame(dict1)
                df_list.append(hfmc_df)
    df_all = pd.concat(df_list, axis=0)
    return df_all

def tran_middle_df_to_ds(df_all,mid_columns = None):
    '''

    :param df_all:
    :param mid_columns: 中间数据对应的列名称，列表形式
    :return:
    '''
    print("该函数有bug，在相邻格点取值完全相同时，会产生nan")
    all_columns = df_all.columns
    if mid_columns is None:

        list_list = method_coluns_dict.values()
        for list1 in list_list:
            eq = True
            for c in range(1,len(list1)+1):
                if all_columns[-c] != list1[-c]:
                    eq = False
            if eq:
                mid_columns = list1
                break

    index_columns = []
    for name in all_columns:
        if name not in mid_columns:
            index_columns.append(name)
    index = []
    for i in range(len(index_columns)):
        index.append(df_all[index_columns[i]].values)
    df_new = pd.DataFrame(df_all[mid_columns].values, index=index, columns=mid_columns)
    df_new.index.names = index_columns
    df_new.drop_duplicates(subset=None, keep='first', inplace=True)
    mid_xr = df_new.to_xarray()
    return mid_xr

def tran_middle_ds_to_df(ds_all):

    values = list(ds_all.data_vars)
    df1 = ds_all[values[0]].to_dataframe()
    df1 = df1.reset_index()
    for i in range(1,len(values)):
        df2 = ds_all[values[i]].to_dataframe()
        df2 = df2.reset_index()
        df1 = pd.merge(df1,df2)
    df_all = df1.dropna()
    return df_all

def middle_ds_sta(sta_all,method,grade_list = None,compare = None,gid = None):
    '''

    :param sta_all:
    :param method:
    :param grade_list:
    :param compare:
    :param gid:
    :return:
    '''
    mid_columns = get_middle_columns(method)
    df_all = middle_df_sta(sta_all,method,grade_list,compare = compare,gid = gid)
    ds_all = tran_middle_df_to_ds(df_all,mid_columns)
    return ds_all

def middle_df_grd(grd_ob, grd_fo, method, grade_list=None, compare=None, marker=None, marker_name=None):
    mid_columns = get_middle_columns(method)
    level_fo = grd_fo["level"].values[0]
    time_fo = grd_fo["time"].values[0]
    dtime = grd_fo["dtime"].values[0]
    fo_name = grd_fo["member"].values[0]

    lats = grd_ob["lat"].values
    lons = grd_ob["lon"].values
    xx,yy = np.meshgrid(lons,lats)
    weight = np.cos(yy*math.pi / 180)
    grid0 = meteva.base.get_grid_of_data(grd_ob)
    grid_fo = meteva.base.get_grid_of_data(grd_fo)
    if grid0.slon != grid_fo.slon or grid0.slat != grid_fo.slat or grid0.dlon != grid_fo.dlon or grid0.dlat != grid_fo.dlat or grid0.nlon != grid_fo.nlon or grid0.nlat != grid_fo.nlat:
        print("the grid of grd_ob is :")
        print("lon:" + str(grid0.glon))
        print("lat:" + str(grid0.glat))
        print("the grid of grd_fo is :")
        print("lon:" + str(grid_fo.glon))
        print("lat:" + str(grid_fo.glat))
        print("the grid of grd_ob is different from the grid of  grd_fo")
        return None

    grid0_ = meteva.base.grid(grid0.glon,grid0.glat)
    grd_weight = meteva.base.grid_data(grid0_,weight)
    #meteva.base.pcolormesh_2d_grid(grd_weight,save_path="a.png")
    if method.__name__.find("_uv") >= 0:
        fo_name = fo_name[2:]

    method_args = {}
    if grade_list is not None:
        method_args["grade_list"] = grade_list

    if compare is not None:
        method_args["compare"] = compare
    df_list = []
    if marker is None:

        ob = grd_ob.values
        fo = grd_fo.values

        if method.__name__.find("_uv")>=0:
            mid_array = method(ob[0,...],fo[0,...], ob[1,...],fo[1,...], **method_args)
        else:
            mid_array = method(ob, fo, **method_args)
            if method.__name__ == "hnh":
                mid_array = mid_array.flatten()
        index_names = ["level","time", "dtime", "member"]
        dict1 = {"level":level_fo,"time": time_fo, "dtime": dtime, "member": fo_name}


        if grade_list is not None:
            index_names.append("grade")
            dict1["grade"] = grade_list

        if method == meteva.method.tc_count:
            if grade_list is None:
                dat1 = mid_array[..., 0]
            else:
                dat1 = np.repeat([mid_array[..., 0]], len(grade_list))
            dict1[mid_columns[0]] = dat1.flatten()
            dict1[mid_columns[1]] = mid_array[..., 1:].flatten()
        else:
            for c in range(len(mid_columns)):
                dict1[mid_columns[c]] = mid_array[..., c].flatten()

        mid_df = pd.DataFrame(dict1)
        df_list.append(mid_df)

    else:
        mark_set = list(set(marker.values.flatten().tolist()))
        mark_name = marker["member"].values[0]
        marker_name_dict = {}
        if  marker_name  is None:
            for mark in mark_set:
                marker_name_dict[mark] = mark
        else:
            for i in range(len(marker_name.index)):
                marker_name_dict[marker_name.iloc[i,0]] = marker_name.iloc[i,1]

        for mark in mark_set:
            if method.__name__.find("_uv")>=0:
                index = np.where(marker.values[0,...] == mark)
                u_ob = grd_ob.values[0, ...]
                u_ob = u_ob[index]
                v_ob = grd_ob.values[1, ...]
                v_ob = v_ob[index]
                u_fo = grd_fo.values[0, ...]
                u_fo = u_fo[index]
                v_fo = grd_fo.values[1, ...]
                v_fo = v_fo[index]
                mid_array = method(u_ob, u_fo, v_ob, v_fo, **method_args)
            else:
                index = np.where(marker.values == mark)
                ob = grd_ob.values[index]
                fo = grd_fo.values[index]
                if method.__name__ in ["tase","tmmsss"]:
                    #目前暂时只支持tase和tmmsss相关的指标的带权重
                    w = grd_weight.values[index]
                    method_args["weight"] = w
                mid_array = method(ob, fo, **method_args)
                if method.__name__ == "hnh":
                    mid_array = mid_array.flatten()

            index_names = ["level","time", "dtime", "member"]
            dict1 = {"level":level_fo,"time": time_fo, "dtime": dtime, "member": fo_name}
            if mark in marker_name_dict.keys():
                dict1[mark_name] = marker_name_dict[mark]
            else:
                dict1[mark_name] = "other"

            if grade_list is not None:
                index_names.append("grade")
                dict1["grade"] = grade_list

            if method == meteva.method.tc_count:
                if grade_list is None:
                    dat1 = mid_array[..., 0]
                else:
                    dat1 = np.repeat([mid_array[..., 0]], len(grade_list))
                dict1[mid_columns[0]] = dat1.flatten()
                dict1[mid_columns[1]] = mid_array[..., 1:].flatten()
            else:
                for c in range(len(mid_columns)):
                    dict1[mid_columns[c]] = mid_array[..., c].flatten()


            mid_df = pd.DataFrame(dict1)
            df_list.append(mid_df)
    df_all = pd.concat(df_list, axis=0)
    return df_all


def middle_ds_grd(grd_ob, grd_fo, method, grade_list=None, compare=None, marker=None, marker_name=None):
    '''

    :param sta_all:
    :param method:
    :param grade_list:
    :param compare:
    :param gid:
    :return:
    '''
    mid_columns = get_middle_columns(method)
    df_all = middle_df_grd(grd_ob,grd_fo,method,grade_list,compare = compare,marker=marker,marker_name=marker_name)
    ds_all = tran_middle_df_to_ds(df_all,mid_columns)
    return ds_all


def get_grid_marker(grid,step = 10):

    marker = meteva.base.grid_data(grid)
    lon = marker["lon"].values
    lat = marker["lat"].values
    if step >=1:
        lon_step = (np.ceil(lon/step)*step).astype(np.int32)
        lat_step = (np.ceil(lat/step)*step).astype(np.int32)
    else:
        lon_step = (np.ceil(lon / step) * step*10).astype(np.int32)
        lat_step = (np.ceil(lat / step) * step*10).astype(np.int32)
    xx,yy = np.meshgrid(lon_step,lat_step)
    if step >= 1:
        zz = yy*1000+xx
    else:
        zz = yy * 10000 + xx
    marker = meteva.base.grid_data(grid,zz)
    meteva.base.set_griddata_coords(marker,member_list=["id"])
    return marker

if __name__ =="__main__":
    pass
    # import pandas as pd
    # path = r"O:\data\hdf\gongbao\rain24h_update12h_station2k\rain24h_update12h_station2k.h5"
    #
    # # #path = r"H:/a.h5"
    # sta_all = pd.read_hdf(path)
    # sta_all = meteva.base.sele_by_para(sta_all, time_range=["2023123108", "2024010208"], dtime_range=[24, 72])
    # print(sta_all)
    # import meteva.product as mpd
    # import meteva.method as mem
    # result = mpd.score(sta_all,mem.ob_fo_precipitation_strength,g = "dtime",plot = "bar",tag = 3,save_path = r"H:\task\a.png")
    # print(result)
    # dfmid = meteva.perspact.middle_df_sta(sta_all,mem.cscs)
    # print(dfmid)
    # result = meteva.perspact.score_df(dfmid,mem.ob_fo_precipitation_strength,g = ["member","dtime"],plot = "bar",tag = 3,save_path = r"H:\task\b.png")
    # print(result)
    # print()


    # sta_all = meteva.base.sele_by_para(sta_all,time_range = ["2023062008","2023062408"],dtime_range = [24,72])
    #
    # print(sta_all.iloc[:,6].values)
    # sta_all1 = sta_all.copy()
    # sta_all1.iloc[:,6] = 0
    # sta_all1.iloc[sta_all.iloc[:,6].values >=10 , 6] = 1
    # sta_all1.iloc[:, 7:] = sta_all.iloc[:, 7:].values /50
    # values = sta_all1.iloc[:,7:].values
    # values[values>1] = 1
    # sta_all1.iloc[:,7:] = values
    # sta_all1.to_hdf(r"H:\test_data\input\mps\rain24_10m_probability.h5","df")

    #method = get_middle_columns(meteva.method.nasws_uv)
    #print(method)

    # sta_all1 = pd.read_hdf(r"H:\test_data\input\mps\rain24_10m_probability.h5")
    # df = middle_df_sta(sta_all1,meteva.method.hfmc_multi,grade_list=[0.3,0.5])
    # print(df)
    # score = meteva.perspact.score_df(df,meteva.method.ts_multi,g = ["member","grade"],plot="line",
    #                                   save_path=r"H:\test_data\output\mps\ts.png")
    # print(score)
    # score = meteva.product.score(sta_all1,meteva.method.ts_multi,grade_list=[0.3,0.5])
    # print(score)


import numpy as np


def fss2dfun(s_py, s_px, subset=None, verbose=False):
    id1 = ~np.isnan(s_py)
    id2 = ~np.isnan(s_px)
    if verbose:
        print("Finding the numbers of non-missing values for each field.\n")
    if subset is None:
        n1 = np.sum(id1)
        n2 = np.sum(id2)
        nxy = np.sum(id1 & id2)
        if verbose:
            print(nxy, " total number of non-missing points.\n")
        num = np.sum(((s_py - s_px) ** 2)) / nxy
        if verbose:
            print("MSE = ", num, "\n")
        denom = np.sum([[y ** 2 for y in x] for x in s_py]) / n1 + np.sum([[y ** 2 for y in x] for x in s_px]) / n2
    else:
        num = np.mean((s_py[subset] - s_px[subset]) ** 2)
        denom = np.mean((s_py[subset]) ** 2 + (s_px[subset]) ** 2)

    if verbose:
        print("Reference MSE = ", denom, "\n")
    return 1 - num / denom



# -*-coding:utf-8-*-
import numpy as np
import pandas as pd
from . import loc_list_setup as lls
#import meteva.method.space.baddeley_binary_image_metric.lib.loc_list_setup as lls
from meteva.method.space.baddeley_binary_image_metric.lib.distmap import *
from .distfun import updateMatrix

def locperf(X, Y, which_stats=None, alpha=0.1, k=4, distfun = "distmapfun", a=None, **args):
    if which_stats is None:
        which_stats = ["qdmapdiff", "med", "msd", "ph", "fom", "minsep"]
    out = lls.loc_list_setup(a, which_stats, nthresh=1)
    window = boundingbox(X, Y)
    X = rebound(X, window)
    Y = rebound(Y, window)
    # dY = ndimage.morphology.distance_transform_edt(~(Y['m'] == 1) + 0)
    # dX = ndimage.morphology.distance_transform_edt(~(X['m'] == 1) + 0)
    z = np.array(~(X['m'] == 1) + 0, np.uint8)
    dX = updateMatrix(np.array(~(X['m'] == 1) + 0))
    dY = updateMatrix(np.array(~(Y['m'] == 1) + 0))
    # dY = cv2.distanceTransform(np.array(~(Y['m'] == 1) + 0, np.uint8), cv2.DIST_L2, 3, dstType=cv2.CV_32F)
    # dX = cv2.distanceTransform(np.array(~(X['m'] == 1) + 0, np.uint8), cv2.DIST_L2, 3, dstType=cv2.CV_32F)
    if not Y['m'].max() == 1:
        dY = np.unique(np.array(dY))
        dYcheck = False
    else:
        dYcheck = True
    if not X['m'].max() == 1:
        dXcheck = False
    else:
        dXcheck = True
    if "med" in which_stats or "msd" in which_stats or "fom" in which_stats or "minsep" in which_stats or "ph" in which_stats:
        if dYcheck:
            Z = (np.array(dY))[(X["m"] == 1)]
        elif X['m'].max() == 1:
            Z = dY
        else:
            Z = 0
        if dXcheck:
            Zother = np.array(dX)[(Y["m"] == 1)]
        elif Y['m'].max() == 1:
            Zother = dX
        else:
            Zother = 0
    if "msd" in which_stats or "fom" in which_stats:
        Z2 = Z ** 2
        Zother2 = Zother ** 2
        if "fom" in which_stats:
            if dYcheck and dXcheck:
                N = max(np.sum(X["m"]), np.sum(Y["m"]))
            elif dYcheck and not dXcheck:
                N = np.sum(Y["m"])
            elif not dYcheck and dXcheck:
                N = np.sum(X["m"])
            else:
                N = 1e+16
    if "ph" in which_stats:
        if k < 1 or k <= np.sum(X["m"]) or k <= np.sum(Y["m"]):
            if dXcheck or dYcheck:
                if k >= 1:
                    if k > np.sum(X["m"]):
                        out['ph'] = np.sort(Zother.flatten())[::-1][k]
                    elif k > np.sum(Y["m"]):
                        out['ph'] = np.sort(Z.flatten())[::-1][k]
                    else:
                        out['ph'] = max(np.sort(Zother.flatten())[::-1][k], np.sort(Z.flatten())[::-1][k])
                elif 0 <= k < 1:
                    out['ph'] = max(np.percentile(Z, k), np.percentile(Zother, k))
                else:
                    out['ph'] = None
            else:
                out['ph'] = max((sorted(Z, reverse=True))[1], sorted(Zother, reverse=True)[1])
        else:
            out['ph'] = None
    if "qdmapdiff" in which_stats:
        diffXY = sorted(abs(dX - dY).T.flatten(), reverse=True)
        if "qdmapdiff" in which_stats:
            if dXcheck or dYcheck:
                if k >= 1:
                    out['qdmapdiff'] = diffXY[k]
                elif 0 <= k < 1:
                    out['qdmapdiff'] = pd.DataFrame(diffXY).quantile(k)
                else:
                    out['qdmapdiff'] = None
            else:
                out['qdmapdiff'] = diffXY[1]
    if "med" in which_stats:
        out['medMiss'] = np.mean(Z)
        out['medFalseAlarm'] = np.mean(Zother)
    if "msd" in which_stats:
        out['msdMiss'] = np.mean(Z2)
        out['msdFalseAlarm'] = np.mean(Zother2)
    if "fom" in which_stats:
        out['fom'] = sum(1 / (1 + alpha * Z2)) / N
    if "minsep" in which_stats:
        if Z.size != 0:
            out['minsep'] = np.min(Z)
        else:
            out['minsep'] = None
    return out



import numpy as np


def hoods2dPrep(input_object, pe=None, levels=None, max_n=None, smooth_fun="hoods2dsmooth", smooth_params=None):
    out = input_object
    data = input_object['X']
    '''
    xdim = [0] * len(data)
    for i in range(0, len(data) - 1, 1):
        xdim[i] = data[i].shape[0]
    '''
    xdim = np.shape(data)
    if levels is None:
        if max_n is None:
            max_n = 2 * max(xdim) - 1
        else:
            if max_n % 2 == 0:
                max_n = max_n - 1
            if max_n > 2 * max(xdim) - 1:
                print("fss2d: max_n must be less than 2N-1, where N is " + str(max(xdim)))
                return
        if max_n < 1:
            print("fss2d: max_n must be a positive integer.")
            return
        levels = np.arange(1, max_n+2, 2)

    out["levels"] = levels
    out["max_n"] = max_n
    out["smooth_fun"] = smooth_fun
    out["smooth_params"] = smooth_params
    if pe is None:
        pe = [1 / item ** 2 for item in levels]
    if len(pe) == 1:
        pe = [pe] * len(levels)
    out["pe"] = pe
    return out



# -*-coding:utf-8-*-
import math
import copy
import numpy as np
from . import data_pre
import meteva

def estimate_negatives(look):
    #根据 目标间平均距离估算negative的数量
    pass

def feature_table(look):
    #x = copy.deepcopy(look)
    out = {}
    grid0 = look["grid"]
    zeros = np.zeros((grid0.nlat,grid0.nlon))
    label_list_ob = look["label_list_ob"]
    for id in label_list_ob:
        index = look["grd_ob_features"][id]
        min_y = np.min(index[0])
        max_y = np.max(index[0])+1

        min_x = np.min(index[1])
        max_x = np.max(index[1])+1

        zeros[min_y:max_y,min_x:max_x] = 1

    area = np.sum(zeros) + 1e-6
    nob_all = int(len(label_list_ob) * grid0.nlat * grid0.nlon / area)+1

    zeros = np.zeros((grid0.nlat,grid0.nlon))
    label_list_fo = look["label_list_fo"]
    for id in label_list_fo:
        index = look["grd_fo_features"][id]
        min_y = np.min(index[0])
        max_y = np.max(index[0]) + 1

        min_x = np.min(index[1])
        max_x = np.max(index[1]) + 1

        zeros[min_y:max_y, min_x:max_x] = 1

    area = np.sum(zeros) + 1e-6
    nfo_all = int(len(label_list_fo) * grid0.nlat * grid0.nlon / area) + 1

    n_all = max(nob_all,nfo_all)
    label_list_matched = look["label_list_matched"]
    hit = len(label_list_matched)
    unmatched = look['unmatched']
    miss = len(unmatched["ob"])
    false_alarm = len(unmatched["fo"])
    correct_negetive = n_all - (hit + miss + false_alarm)
    if correct_negetive<0: correct_negetive = 0

    hfmc = np.array([hit,false_alarm,miss,correct_negetive])

    ets = meteva.method.ets_hfmc(hfmc)
    pod = meteva.method.pod_hfmc(hfmc)
    pofd = meteva.method.pofd_hfmc(hfmc)
    far = meteva.method.far_hfmc(hfmc)
    hss = meteva.method.hss_yesorno_hfmc(hfmc)

    theta = {'ets': ets, 'pod': pod, 'pofd': pofd, 'far': far, 'hss': hss}
    tab = {'Hits': hit, 'Misses': miss, 'False alarms': false_alarm, 'Correct negatives': correct_negetive}
    out['contingency_table_yesorno'] = tab
    out['score'] = theta
    return out

def feature_table_old(look, fudge=1e-08, hits_random=None, correct_negatives=None, fA=0.05):

    x = copy.deepcopy(look)
    out = {}
    #m = x["unmatched"]["matches"]
    #hits = m.shape[0]
    m = x["matches"]

    #if (x['match_type'] == 'deltamm'):
    #    hits = m['ob'].shape[0]
    if x['match_type'] in ['centmatch' ,'deltamm' ,'minboundmatch',"MergeForce","unimatch"]:
        hits = len(m[:, 0])
    miss = get_length(x['unmatched']['ob'])
    fa = get_length(x['unmatched']['fo'])
    #No = x['Xlabeled'].max()
    #Nf = x['Ylabeled'].max()
    Xfeats = data_pre.pick_labels(x['grd_ob_features'])
    Yfeats = data_pre.pick_labels(x['grd_fo_features'])
    No = len(Xfeats)
    Nf = len(Yfeats)

    if correct_negatives is None:
        bigD = (1 - fA) * No / fA - Nf
        if bigD < 0:
            print(
                "FeatureTable: attempted to estimate a value for correct.negatives," 
                "but got a negative answer.  Setting to NA.")
            bigD = None
            bigD = 1
        elif bigD == 0:
            bigD = fudge
    else:
        bigD = correct_negatives
    if hits_random is None:
        hits_random = (Nf * No) / (Nf + miss + bigD)

    #print(bigD)
    denom = Nf + miss - hits_random
    if denom == 0:
        denom = fudge
    GSS = (hits - hits_random) / denom
    s = (hits + miss) / bigD
    POD = hits / (hits + miss + fudge)
    SH2 = POD * (1 - POD) / (hits + miss + fudge)
    POD_se = math.sqrt(SH2)
    FArate = fa / (fa + bigD)
    SF2 = FArate * (1 - FArate) / (fa + bigD)
    FArate_se = math.sqrt(SF2)
    FAR = fa / (hits + fa + fudge)
    FAR_se = math.sqrt(
        (FAR ** 4) * ((1 - POD) / (hits + fudge) + (1 - POD) / (fa + fudge)) * (hits ** 2 / (fa ** 2 + fudge)))
    HSS = 2 * (hits * bigD - fa * miss) / ((hits + miss) * (miss + bigD) + (hits + fa) * (fa + bigD) + fudge)
    SHSS2 = SF2 * (HSS ** 2) * (1 / (POD - FArate + fudge) + (1 - s) * (1 - 2 * s)) ** 2 + SH2 * (HSS ** 2) * (
            1 / (POD - FArate + fudge) - s * (1 - 2 * s)) ** 2
    HSS_se = math.sqrt(SHSS2)
    if 2 - HSS == 0:
        GSS_se = math.sqrt(4 * SHSS2 / fudge)
    else:
        GSS_se = math.sqrt(4 * SHSS2 / ((2 - HSS) ** 4))
    theta = {'ets': GSS, 'pod': POD, 'pofd': FArate, 'far': FAR, 'hss': HSS}
    theta_se = {'ETS': GSS_se, 'POD': POD_se, 'POFD': FArate_se, 'FAR': FAR_se, 'HSS': HSS_se}
    #out['se'] = theta_se
    tab = {'Hits': hits, 'Misses': miss, 'False alarms': fa, 'Correct negatives': bigD}
    out['contingency_table_yesorno'] = tab
    out['score'] = theta
    return out

def get_length(x):
    data_type = type(x).__name__
    # 此处应该将调用方法中的‘None’改为None
    if data_type == "str" and x == 'NULL':
        return 0
    elif data_type == "int" or data_type == "float":
        return 1
    elif data_type == "DataFrame" or data_type == "ndarray":
        return x.size
    #添加set类型
    elif data_type == "set" or data_type == "list":
        return len(x)
    else:
        print("传入类型错误")
        raise Exception("传入类型错误")





from . import  as_unitname
from . import  datagrabber
from . import  hoods2dPrep
from . import  im
from . import kernel2dmeitsjer
from . import kernel2dsmooth
from . import make_spatialVx
from . import thresholder
from . import thresholder
from . import vxstats



from . import time_tools
from . import math_tools
from .frprmn2 import *
from . import path_tools
from . import time_tools
from . import color_tools
from . import copy_tools
from . import plot_tools
from . import grib_tools
from . import maskout
from .path_tools import get_path
from .path_tools import creat_path

from .copy_tools import copy_m4_to_nc,copy_data
from .time_tools import all_type_time_to_datetime,all_type_time_to_time64,all_type_time_to_str
from .time_tools import all_type_timedelta_to_timedelta64,all_type_timedelta_to_timedelta
from .grib_tools import grib_to_nc
from .station_tools import get_station_id_name_dict,station_id_name_dict,station_name_id_dict,find_station_id_by_city_name,get_station_format_province_set
from .process_tools import multi_run
from .plot_tools import contourf_2d_grid,pcolormesh_2d_grid,scatter_sta,bar,plot,mesh,set_customized_shpfile_list,add_scatter,bar_line,myheatmap,contourf,barbs_grid_wind
from .color_tools import cmaps,def_cmap_clevs,merge_cmap_clevs,get_seprated_rgb_method1,get_seprated_rgb_method2,set_plot_color_dict_method0,set_plot_color_dict_method1,set_plot_color_dict_method2
from .maskout import *
from .plot_tools_adv import *
from .plot_tools import contourf_xz,contourf_yz,lineh,contourf_xy,mesh_xy,contourf


import datetime
import numpy as np
import re

#所有类型的时间转换为time64
def all_type_time_to_time64(time0):
    if isinstance(time0,np.datetime64):
        return time0
    elif isinstance(time0,datetime.datetime):
        return np.datetime64(time0)
    elif type(time0) == str:
        return str_to_time64(time0)
    elif isinstance(time0,int):
        time1 = datetime.datetime.utcfromtimestamp(time0 / 1000000000)
        np.datetime64(time1)
        return np.datetime64(time1)
    else:
        print("时间类型不识别")
        return None

def all_type_time_to_datetime(time0):
    if isinstance(time0,int):
        time1 = datetime.datetime.utcfromtimestamp(time0 / 1000000000)
        return time1
    time64 = all_type_time_to_time64(time0)
    time1 = time64.astype(datetime.datetime)
    if isinstance(time1,int):
        time1 = datetime.datetime.utcfromtimestamp(time1/1000000000)
    return time1

def all_type_time_to_str(time0):
    time1 = all_type_time_to_time64(time0)
    return time_to_str(time1)


#所有的timedelta类型的数据转为timedelta64类型的时间格式
def all_type_timedelta_to_timedelta64(timedelta0):
    if isinstance(timedelta0,np.timedelta64):
        return timedelta0
    elif isinstance(timedelta0,datetime.timedelta):
        return np.timedelta64(timedelta0)
    elif type(timedelta0) == str:
        timedelta1 = str_to_timedelta(timedelta0)
        return np.timedelta64(timedelta1)
    else:
        print("时效类型不识别")
        return None

def all_type_timedelta_to_timedelta(timedelta0):
    if isinstance(timedelta0,datetime.timedelta):
        return timedelta0
    elif isinstance(timedelta0,np.timedelta64):
        seconds = int(timedelta0 / np.timedelta64(1, 's'))
        timedelta1 = datetime.timedelta(seconds = seconds)
        return timedelta1
    elif type(timedelta0) == str:
        timedelta1 = str_to_timedelta(timedelta0)
        return timedelta1
    else:
        print("时效类型不识别")
        return None

#str类型的时间转换为timedelta64类型的时间
def str_to_timedelta(timedalta_str):
    num_str = ''.join([x for x in timedalta_str if x.isdigit()])
    num = int(num_str)
    # 提取出dtime_type类型
    TIME_type = re.findall(r"\D+", timedalta_str)[0].lower()
    if TIME_type == 'h':
        return datetime.timedelta(hours=num)
    elif TIME_type == 'd':
        return datetime.timedelta(days=num)
    elif TIME_type == 'm':
        return datetime.timedelta(minutes=num)
    else:
        print("输入的时效格式不识别")
        return None

#str类型的时间转换为time64类型的时间
def str_to_time64(time_str):
    str1 = ''.join([x for x in time_str if x.isdigit()])
    # 用户输入2019041910十位字符，后面补全加0000，为14位统一处理
    if len(str1) == 4:
        str1 += "0101000000"
    elif len(str1) == 6:
        str1 +="01000000"
    elif len(str1) == 8:
        str1 +="000000"
    elif len(str1) == 10:
        str1 +="0000"
    elif len(str1) == 12:
        str1 +="00"
    elif len(str1) == 14:
        str1 = time_str
    else:
        print("输入日期格式不识别，请检查！")

    # 统一将日期变为datetime类型
    time = datetime.datetime.strptime(str1, '%Y%m%d%H%M%S')
    time64 = np.datetime64(time)
    return time64



#datetime64类型的数据转换为str类型
def time_to_str(time):
    if isinstance(time,np.datetime64):
        str1 = str(time).replace("-", "").replace(" ", "").replace(":", "").replace("T", "")[0:14]
    else:
        str1 = time.strftime("%Y%m%d%H%M%S")
    return str1


#字符转换为datetime
def str_to_time(str0):
    num = ''.join([x for x in str0 if x.isdigit()])
    # 用户输入2019041910十位字符，后面补全加0000，为14位统一处理
    if len(num) == 4:
        num += "0101000000"
    elif len(num) == 6:
        num += "01000000"
    elif len(num) == 8:
        num += "000000"
    elif len(num) == 10:
        num += "0000"
    elif len(num) == 12:
        num += "00"
    elif len(num) > 12:
        num = num[0:12]
    else:
        print("输入日期有误，请检查！")
    # 统一将日期变为datetime类型
    return datetime.datetime.strptime(num, '%Y%m%d%H%M%S')


def get_dtime_of_path(path_model,path):
    ttt_index = path_model.find("TTT")
    if (ttt_index >= 0):
        ttt = int(path[ttt_index:ttt_index + 3])
    else:
        ttt = 0
    return ttt
def get_time_of_path(path_model,path):
    yy_index = path_model.find("YYYY")
    if  yy_index < 0:
        yy_index = path_model.find("YY")
        if(yy_index <0):
            yy = 2000
        else:
            yy = int(path[yy_index: yy_index + 2]) + 2000
    else:
        yy = int(path[yy_index: yy_index+4])

    mm_index = path_model.find("MM")
    if(mm_index >=0):
        mm = int(path[mm_index:mm_index+2])
    else:
        mm = 1

    dd_index = path_model.find("DD")
    if(dd_index>=0):
        dd = int(path[dd_index:dd_index + 2])
    else:
        dd = 1
    hh_index = path_model.find("HH")
    if(hh_index>=0):
        hh = int(path[hh_index:hh_index + 2])
    else:
        hh = 0
    ff_index = path_model.find("FF")
    if(ff_index>=0):
        ff = int(path[ff_index:ff_index + 2])
    else:
        ff = 0
    ss_index = path_model.find("SS")
    if(ss_index>=0):
        ss = int(path[ss_index:ss_index + 2])
    else:
        ss = 0
    return datetime.datetime(yy,mm,dd,hh,ff,ss)




import meteva
import copy
import datetime
import math
import time
import pandas as pd
import numpy as np




def get_time_str_one_by_one(time1,time0 = None,row = 1):

    if meteva.base.language =="ch":
        if row == 3:
            if time0 is None:
                time2 = meteva.base.tool.time_tools.all_type_time_to_datetime(time1)
                if  time2.minute == 0:
                    time_str = time2.strftime('%H{h}\n%d{d}\n%Y{y}%m{m}').format(y='年', m='月', d='日',h = "时")
                else:
                    time_str = time2.strftime('%H{h}%M{mi}\n%m{m}%d{d}\n%Y{y}').format(y='年', m='月', d='日',h='时',mi = '分')
            else:
                time00 = meteva.base.tool.time_tools.all_type_time_to_datetime(time0)
                time2 = meteva.base.tool.time_tools.all_type_time_to_datetime(time1)
                if time2.year != time00.year:
                    if time2.hour == 0 and time2.minute == 0:
                        time_str = time1.strftime('%d{d}\n%m{m}\n%Y{y}').format(y='年', m='月', d='日')
                    elif time1.minute == 0:
                        time_str = time1.strftime('%H{h}\n%d{d}\n%Y{y}%m{m}').format(y='年', m='月', d='日',h='时')
                    else:
                        time_str = time1.strftime('%M{mi}\n%H{h}\n%Y{y}%m{m}%d{d}').format(y='年', m='月', d='日',h='时',mi = '分')
                elif time2.month != time00.month:
                    if time2.hour == 0 and time2.minute == 0:
                        time_str = time2.strftime('%d{d}\n%m{m}').format(m='月', d='日')
                    elif time2.minute == 0:
                        time_str = time2.strftime('%H{h}\n%d{d}\n%m{m}').format(m='月', d='日',h='时')
                    else:
                        time_str = time2.strftime('%M{mi}\n%H{h}\n%m{m}%d{d}').format(m='月', d='日',h='时',mi = '分')
                elif time2.day != time00.day:
                    if time2.hour == 0 and time2.minute == 0 and time00.hour ==0:
                        time_str = time2.strftime('%d{d}').format(d='日')
                    elif time2.minute == 0:
                        time_str = time2.strftime('%H{h}\n%d{d}').format(d='日',h='时')
                    else:
                        time_str = time2.strftime('%M{mi}\n%H{h}\n%d{d}').format(d='日',h='时',mi = '分')
                elif time2.hour != time00.hour:
                    if time2.minute == 0:
                        time_str = time2.strftime('%H{h}').format(h='时')
                    else:
                        time_str = time2.strftime('%M{mi}\n%H{h}').format(h='时',mi = '分')
                else:
                    time_str = time2.strftime("%M")+"分"
        elif row == 2:
            if time0 is None:
                time2 = meteva.base.tool.time_tools.all_type_time_to_datetime(time1)
                if time2.hour == 0 and time2.minute == 0:
                    time_str = time2.strftime('%d{d}\n%Y{y}%m{m}').format(y='年', m='月', d='日')
                elif time2.minute == 0:
                    time_str = time2.strftime('%H{h}\n%d{d}\n%Y{y}%m{m}').format(y='年', m='月', d='日',h='时')
                else:
                    time_str = time2.strftime('%M{mi}\n%H{h}\n%Y{y}%m{m}%d{d}').format(y='年', m='月', d='日',h='时',mi = '分')
            else:
                time00 = meteva.base.tool.time_tools.all_type_time_to_datetime(time0)
                time2 = meteva.base.tool.time_tools.all_type_time_to_datetime(time1)
                if time2.year != time00.year:
                    if time2.hour == 0 and time2.minute == 0:
                        time_str = time1.strftime('%d{d}\n%Y{y}%m{m}').format(y='年', m='月', d='日')
                    elif time1.minute == 0:
                        time_str = time1.strftime('%H{h}\n%d{d}\n%Y{y}%m{m}').format(y='年', m='月', d='日',h='时')
                    else:
                        time_str = time1.strftime('%M{mi}\n%H{h}\n%Y{y}%m{m}%d{d}').format(y='年', m='月', d='日',h='时',mi = '分')
                elif time2.month != time00.month:
                    if time2.hour == 0 and time2.minute == 0:
                        time_str = time2.strftime('%d{d}\n%m{m}').format(m='月', d='日')
                    elif time2.minute == 0:
                        time_str = time2.strftime('%H{h}\n%m{m}%d{d}').format(m='月', d='日',h='时')
                    else:
                        time_str = time2.strftime('%M{mi}\n%H{h}\n%m{m}%d{d}').format(m='月', d='日',h='时',mi = '分')
                elif time2.day != time00.day:
                    if time2.hour == 0 and time2.minute == 0:
                        time_str = time2.strftime('%d{d}').format(d='日')
                    elif time2.minute == 0:
                        time_str = time2.strftime('%H{h}\n%d{d}').format(d='日',h='时')
                    else:
                        time_str = time2.strftime('%M{mi}\n%H{h}\n%d{d}').format(d='日',h='时',mi = '分')
                elif time2.hour != time00.hour:
                    if time2.minute == 0:
                        time_str = time2.strftime('%H{h}').format(h='时')
                    else:
                        time_str = time2.strftime('%M{mi}\n%H{h}').format(h='时',mi = '分')
                else:
                    time_str = time2.strftime("%M")+"分"
        else:
            if time0 is None:
                time2 = meteva.base.tool.time_tools.all_type_time_to_datetime(time1)
                if time2.hour == 0 and time2.minute == 0:

                    time_str = time2.strftime('%Y{y}%m{m}%d{d}').format(y='年', m='月', d='日')
                elif time2.minute == 0:
                    time_str = time2.strftime('%Y{y}%m{m}%d{d}%H{h}').format(y='年', m='月', d='日', h='时')
                else:
                    time_str = time2.strftime('%Y{y}%m{m}%d{d}%H{h}%M{mi}').format(y='年', m='月', d='日', h='时', mi='分')
            else:
                time00 = meteva.base.tool.time_tools.all_type_time_to_datetime(time0)
                time2 = meteva.base.tool.time_tools.all_type_time_to_datetime(time1)
                if time2.year != time00.year:
                    if time2.hour == 0 and time2.minute == 0:
                        time_str = time1.strftime('%Y{y}%m{m}%d{d}').format(y='年', m='月', d='日')
                    elif time1.minute == 0:
                        time_str = time1.strftime('%Y{y}%m{m}%d{d}%H{h}').format(y='年', m='月', d='日', h='时')
                    else:
                        time_str = time1.strftime('%Y{y}%m{m}%d{d}%H{h}%M{mi}').format(y='年', m='月', d='日', h='时', mi='分')
                elif time2.month != time00.month:
                    if time2.hour == 0 and time2.minute == 0:
                        time_str = time2.strftime('%m{m}%d{d}').format(m='月', d='日')
                    elif time2.minute == 0:
                        time_str = time2.strftime('%m{m}%d{d}%H{h}').format(m='月', d='日', h='时')
                    else:
                        time_str = time2.strftime('%m{m}%d{d}%H{h}%M{mi}').format(m='月', d='日', h='时', mi='分')
                elif time2.day != time00.day:
                    if time2.hour == 0 and time2.minute == 0:
                        time_str = time2.strftime('%d{d}').format(d='日')
                    elif time2.minute == 0:
                        time_str = time2.strftime('%d{d}%H{h}').format(d='日', h='时')
                    else:
                        time_str = time2.strftime('%d{d}%H{h}%M{mi}').format(d='日', h='时', mi='分')
                elif time2.hour != time00.hour:
                    if time2.minute == 0:
                        time_str = time2.strftime('%H{h}').format(h='时')
                    else:
                        time_str = time2.strftime('%H{h}%M{mi}').format(h='时', mi='分')
                else:
                    time_str = time2.strftime("%M")+"分"
    else:
        if row >=1:
            if time0 is None:
                time2 = meteva.base.tool.time_tools.all_type_time_to_datetime(time1)
                time_str = time2.strftime('%H{h}:%M{mi}\n%m{m}-%d{d}\n%Y{y}').format(y='', m='', d='', h='',
                                                                                       mi='')
            else:
                time00 = meteva.base.tool.time_tools.all_type_time_to_datetime(time0)
                time2 = meteva.base.tool.time_tools.all_type_time_to_datetime(time1)
                if time2.year != time00.year:
                    time_str = time2.strftime('%H{h}:%M{mi}\n%m{m}-%d{d}\n%Y{y}').format(y='', m='', d='', h='',
                                                                                        mi='')
                else:
                    time_str = time2.strftime('%H{h}:%M{mi}\n%m{m}-%d{d}').format(m='', d='', h=':',
                                                                                        mi='')
        else:
            if time0 is None:
                time2 = meteva.base.tool.time_tools.all_type_time_to_datetime(time1)
                time_str = time2.strftime('%Y{y}-%m{m}-%d{d} %H{h}:%M{mi}').format(y='', m='', d='', h='', mi='')
            else:
                time2 = meteva.base.tool.time_tools.all_type_time_to_datetime(time1)
                time_str = time2.strftime('%Y{y}-%m{m}-%d{d} %H{h}:%M{mi}').format(y='', m='', d='', h='', mi='')

    return time_str

def get_time_str_list(time_list,row = 1):
    str1 = get_time_str_one_by_one(time_list[0],None,row)
    time_str_list = [str1]
    for i in range(1,len(time_list)):
        time0 = meteva.base.time_tools.all_type_time_to_datetime(time_list[i])
        time_1 =  meteva.base.time_tools.all_type_time_to_datetime(time_list[i-1])
        time_str_list.append(get_time_str_one_by_one(time0,time_1,row))
    return time_str_list

def get_dayofyear_str_list(dayofyear_list):
    dati0 = datetime.datetime(2019,1,1,0)
    time_list = []
    for i in range(len(dayofyear_list)):
        dati1 = dati0 + datetime.timedelta(days=int(dayofyear_list[i]) - 1)
        time_list.append(dati1)

    str_list = get_time_str_list(time_list,row=2)
    str0 = str_list[0]
    ind = str0.find("年")
    str0_year = str0[ind-4:ind+1]
    str0_new = str0.replace(str0_year,"")
    str_list[0] = str0_new
    return str_list


def get_xun_str_list(xun_list):
    xun_strs =[]
    for i in range(12):
        for j in range(3):
            if j==0:
                str1 = ""+str(j+1)+"旬\n"+str(i+1)+"月"
            else:
                str1 = ""+str(j+1)+"旬"
            xun_strs.append(str1)
    str_list = []
    for k in xun_list:
        str_list.append(xun_strs[k-1])
    return str_list


def get_save_path(save_dir,method,group_by,group_list,model_name = "",type = "",discription = ""):

    if discription is None:
        discription = ""
    else:
        discription ="_"+discription
    if save_dir is None:
        save_path = None
    else:
        save_dir = save_dir.replace("\\", "/")
        if group_by is None:
            save_path = save_dir + "/"+method.__name__+"_" +model_name+discription+type
        else:
            print(model_name)
            group_list_str = str(group_list)
            group_list_str = group_list_str.replace(":","").replace(" ","")
            save_path = save_dir + "/" +method.__name__+"_" + model_name + "_"+ group_by + group_list_str+discription+type
    return save_path



def get_title_from_dict(method,s,g,group_list,model_name,title = None):

    if title is not None:
        title1 = title
    else:

        if isinstance(method,str):
            method_str = method
        else:
            #method_str =  method.__defaults__[-1]


            method_para = method.__code__.co_varnames[:method.__code__.co_argcount]

            #print(method_para)
            if "title" in method_para:
                len1 =   len(method_para)
                len2 = len(method.__defaults__)
                dlen = len1 - len2
                for ii in range(len2):
                    if method_para[ii+dlen] == "title":
                        method_str = method.__defaults__[ii]
            else:
                method_str = method.__name__.upper()


        s_str = ""
        if s is not None:

            r = dict(s)
            if "drop_last" in r.keys():
                del r["drop_last"]
            if "member" in r.keys():
                del r["member"]

            if "grid" in r.keys():
                grid0 = r["grid"]
                del r["grid"]
                grid_str = "grid:[" + str(grid0.slon)+"-"+str(grid0.elon)+"°E, " + str(grid0.slat) + "-" + str(grid0.elat)+"°N]"
                if len(r.keys())>0:
                    s_str = str(r)+""
                    s_str = s_str.replace("{","").replace("}","")
                    s_str +=","+grid_str
                    s_str = "\n{" + s_str+"}"
                else:
                    s_str ="\n{" + grid_str +"}"
            if "id" in r.keys():
                id0 = r["id"]
                del r["id"]
                if id0 == meteva.base.IV or np.isnan(id0):
                    s_str = ""
                else:
                    if id0 in meteva.base.tool.station_id_name_dict.keys():
                        id_str = "id:"+ str(id0) +"(" + meteva.base.tool.station_id_name_dict[id0]+")"
                    else:
                        id_str = "id:" + str(id0)
                    if len(r.keys()) > 0:
                        s_str = str(r) + ""
                        s_str = s_str.replace("{", "").replace("}", "")
                        s_str = id_str+ "," +s_str
                        s_str = "\n{" + s_str + "}"
                    else:
                        s_str = "\n{" + id_str + "}"
            else:
                if len(r.keys())>0:
                    s_str = "\n"+str(r)+""


        group_name = ""
        if g is not None:
            group_name = "\n("+g+"="
            if not isinstance(g,list):
                group_list = [group_list]
            g_num = len(group_list)
            if g in ["time","ob_time","day","ob_day"]:
                #判断是否等间距
                if g_num == 1:
                    time_str = get_time_str_one_by_one(group_list[0])
                    group_name += time_str
                elif g_num == 2:
                    time_str1 = get_time_str_one_by_one(group_list[0])
                    time_str2 = get_time_str_one_by_one(group_list[1],group_list[0])
                    group_name += time_str1+"|"+time_str2
                elif g_num == 3:
                    time_str1 = get_time_str_one_by_one(group_list[0])
                    time_str2 = get_time_str_one_by_one(group_list[1],group_list[0])
                    time_str3 = get_time_str_one_by_one(group_list[2], group_list[1])
                    group_name += time_str1+"|"+time_str2 +"|"+time_str3
                else:
                    #判断时间是否等间距
                    time_str1 = get_time_str_one_by_one(group_list[0])
                    time_str2 = get_time_str_one_by_one(group_list[-1],group_list[0])
                    group_name += time_str1 + "至" + time_str2
            else:

                if g_num < 5:
                    for i in range(g_num):
                        loc = group_list[i]
                        if type(loc) == str:
                            group_name += loc
                        else:
                            group_name += str(loc)

                        if i < len(group_list) - 1:
                            group_name += "|"
                else:
                    loc = group_list[0]
                    if type(loc) == str:
                        group_name += loc +"|"+ group_list[1]+"|...|"+group_list[-1]
                    else:
                        group_name += str(loc)+"|"+ str(group_list[1])+"|...|"+str(group_list[-1])
            group_name = group_name + ")"
        if model_name is not None:
            model_name = str(model_name)
        else:
            model_name = ""
        title1 = method_str + model_name + s_str +group_name

    return title1

def get_title(method,group_by,group_list,model_name,title = None,discription_uni = ""):

    if group_by is None:
        if(title is None):
            title1 = method.__defaults__[-1] + "(" + model_name  + ")" + discription_uni
        else:
            title1 = title+ "(" + model_name  + ")"
    else:
        group_name = group_by + "="

        if not isinstance(group_list,list):
            group_list = [group_list]
        g_num = len(group_list)
        if group_by in ["time","ob_time","day","ob_day"]:
            #判断是否等间距
            if g_num == 1:
                time_str = get_time_str_one_by_one(group_list[0])
                group_name += time_str
            elif g_num == 2:
                time_str1 = get_time_str_one_by_one(group_list[0])
                time_str2 = get_time_str_one_by_one(group_list[1],group_list[0])
                group_name += time_str1+"|"+time_str2
            elif g_num == 3:
                time_str1 = get_time_str_one_by_one(group_list[0])
                time_str2 = get_time_str_one_by_one(group_list[1],group_list[0])
                time_str3 = get_time_str_one_by_one(group_list[2], group_list[1])
                group_name += time_str1+"|"+time_str2 +"|"+time_str3
            else:
                #判断时间是否等间距
                time_str1 = get_time_str_one_by_one(group_list[0])
                time_str2 = get_time_str_one_by_one(group_list[-1],group_list[0])
                group_name += time_str1 + "至" + time_str2
        else:

            if g_num < 5:
                for i in range(g_num):
                    loc = group_list[i]
                    if type(loc) == str:
                        group_name += loc
                    else:
                        group_name += str(loc)

                    if i < len(group_list) - 1:
                        group_name += "|"
            else:
                loc = group_list[0]
                if type(loc) == str:
                    group_name += loc +"|"+ group_list[1]+"|...|"+group_list[-1]
                else:
                    group_name += str(loc)+"|"+ str(group_list[1])+"|...|"+str(group_list[-1])

        if title is not None:
            title1 = title + discription_uni
        else:
            title1 = method.__defaults__[-1] + "(" + model_name + ")" + discription_uni +"\n("+group_name+")"
    return title1


def get_unique_coods(sta):
    begin = time.time()
    nline = len(sta.index)
    #print(nline)
    discription = ""
    if sta["level"].values[0] == sta["level"].values[-1]:
        repete = len(sta["level"].drop_duplicates().index)
        if repete == 1:
            discription += "level=" + str(sta["level"].values[0]) +" "
    #print("level")
    #print(time.time() - begin)
    #判断空间的一致性
    not_unique = True
    if sta["id"].values[0] == sta["id"].values[-1]:
        repete = len(sta["time"].drop_duplicates().index)
        if repete == 1:
            discription += "id=" + str(sta["id"].values[0])+" "
            not_unique = False
    #print("id")
    #print(time.time() - begin)

    time0 = meteva.base.time_tools.all_type_time_to_datetime(sta["time"].values[0])
    time_1 =  meteva.base.time_tools.all_type_time_to_datetime(sta["time"].values[-1])

    #判断时间的一致性
    not_unique = True
    if time0 == time_1:
        repete = len(sta["time"].drop_duplicates().index)
        if repete == 1:
            print(sta["time"].values[0])
            discription += "time=" + get_time_str_one_by_one(sta["time"].values[0])+" "
            not_unique = False
    #print("time")
    #print(time.time() - begin)
    if not_unique:
        #判断是否是同一fo_hour
        if time0.hour == time_1.hour:

            times = pd.Series(0, sta["time"])
            if len(times.index.hour.drop_duplicates()) == 1:
                discription += "hour=" + str(times.index.hour[0])+" "
                not_unique = False

        #print("hour")
        #print(time.time() - begin)

        day_unique = False
        if not_unique:
            #判断是否是同一fo_dayofyear
            #dayofyears = sta['time'].map(lambda x: x.dayofyear)
            if time0.timetuple().tm_yday == time_1.timetuple().tm_yday:
                times = pd.Series(0, sta["time"])
                if len(times.index.dayofyear.drop_duplicates())==1:
                    discription += "dayofyear=" + str(times.index.dayofyear[0])+" "
                    day_unique = True

        #print("day")
        #print(time.time() - begin)
        #如果日期是一致的，就不用判断年月了
        month_unique = False
        if not day_unique:
            #判断是否是同一fo_month
            if time0.month == time_1.month:
                times =  pd.Series(0,sta["time"])
                if len(times.index.month.drop_duplicates())==1:
                    discription += "month=" + str(times.index.month[0])+" "
                    month_unique = True
        #print("month")
        #print(time.time() - begin)
        if not month_unique:
            #判断是否是同一fo_year
            if time0.year == time_1.year:
                times  = pd.Series(0,sta["time"])
                if len(times.index.year.drop_duplicates()) == 1:
                    discription += "year=" + str(times.index.year[0])+" "
    #print(time.time() - begin)
    if discription != "":
        discription = "\n("+discription[0:-1]+")"
    return discription

def get_group_name(group_list_list):
    if group_list_list is None:
        return ["all"]
    group_name = []
    group_list0 = group_list_list[0]
    if isinstance(group_list0,list):
        group0 = group_list0[0]
    else:
        group0 = group_list0
    if isinstance(group0,datetime.datetime):
        #判断是否都为单个时刻
        islist = False
        for group_list in group_list_list:
            if isinstance(group_list,list):
               islist = True
        if not islist:
            group_name = get_time_str_list(group_list_list,row=3)
        else:
            for i in range(len(group_list_list)):
                group_name.append("tg"+str(i))
    else:
        for group_list in group_list_list:
            if isinstance(group_list,list):
                str1 = str(group_list)
                if len(str1)<10:
                    str2 = str1
                else:
                    str2 = str1[0:3]+"..."+str1[-3:]
            else:
                str2 = str(group_list)
            group_name.append(str2)
    return group_name

def get_x_label(groupy_by):
    if groupy_by =="time":
        return "时间(预报起报时间)"
    elif groupy_by == "level":
        return "层次"
    elif groupy_by=="year":
        return "年份(预报起报时间)"
    elif groupy_by=="month":
        return "月份(预报起报时间)"
    elif groupy_by=="day":
        return "日期(预报起报时间)"
    elif groupy_by=="dayofyear":
        return "日期在一年中的排序(预报起报时间)"
    elif groupy_by=="hour":
        return "小时数(预报起报时间)"
    elif groupy_by =="ob_time":
        return "观测时间"
    elif groupy_by=="ob_year":
        return "年份(观测时间)"
    elif groupy_by=="ob_month":
        return "月份(观测时间)"
    elif groupy_by=="ob_day":
        return "日期(观测时间)"
    elif groupy_by=="ob_dayofyear":
        return "日期在一年中的排序(观测时间)"
    elif groupy_by=="ob_hour":
        return "小时数(观测时间)"
    elif groupy_by == "dtime":
        return "时效"
    elif groupy_by == "dday":
        return "预报时效包含的天数"
    elif groupy_by == "dday":
        return "预报时效整除24小时后的余数"



def get_x_ticks(ticks,width,row = 2):
    w_one_tick = 0.4
    max_tick_num = max(int(width / w_one_tick),1)
    tick0 = ticks[0]
    if isinstance(tick0,datetime.datetime) or isinstance(tick0,np.datetime64):
        ticks1 = []
        if  isinstance(tick0,datetime.datetime):
            for tick in ticks:
                ticks1.append(meteva.base.time_tools.all_type_time_to_time64(tick))
        else:
            ticks1 = ticks
        ticks1 = list(set(ticks1))
        ticks1.sort()
        times = np.array(ticks1)
        pd_times = pd.Series(0, index=times)
        hours_pd = list(set(pd_times.index.hour))
        hours_pd.sort()
        dtimes = (times[1:] - times[0:-1])
        dhs =  dtimes / np.timedelta64(1, 'h')
        dhs_set = set(dhs.tolist())
        dh_max = (times[-1] - times[0]) / np.timedelta64(1, 'h')
        # 判断是否为有规律的
        if(dhs.size / len(dhs_set) >3):
            #有规律
            dhs_units = np.array(list(dhs_set))
            dhs_units.sort()
            dhs_u0 = dhs_units[0]
            nt = dh_max/dhs_u0
            sp_rate = int(math.ceil(nt/max_tick_num))
            dhs_u1 = dhs_u0 * sp_rate
            #print(dhs_u1)
            if dhs_u1 <=24:
                if dhs_u1 ==1:
                    hour_list = np.arange(24).tolist()
                elif dhs_u1 <=3:
                    hour_list = np.arange(2,24,3).tolist()
                elif dhs_u1 <=6:
                    hour_list = np.arange(2,24,6).tolist()
                elif dhs_u1 <=12:
                    hour_list = np.arange(8,24,12).tolist()
                elif dhs_u1 <=24:
                    if 8 in hours_pd:
                        hour_list = [8]
                    elif 0 in hours_pd:
                        hour_list = [0]
                    else:
                        hour_list = [hours_pd[0]]

                index1 = np.where(pd_times.index.hour.isin(hour_list))
                if len(index1[0]) == 0:
                    if dhs_u1 == 1:
                        hour_list = np.arange(24).tolist()
                    elif dhs_u1 <= 3:
                        hour_list = np.arange(0, 24, 3).tolist()
                    elif dhs_u1 <= 6:
                        hour_list = np.arange(0, 24, 6).tolist()
                    elif dhs_u1 <= 12:
                        hour_list = np.arange(0, 24, 12).tolist()
                    elif dhs_u1 <= 24:
                        if 0 in hours_pd:
                            hour_list = [0]
                        elif 12 in hours_pd:
                            hour_list = [12]
                        else:
                            hour_list = [hours_pd[0]]
                    #print(hour_list)
                    index1 = np.where(pd_times.index.hour.isin(hour_list))
            else:
                hour_list = hours_pd
                index1 = np.where(pd_times.index.hour.isin(hour_list))
                index1 = index1[0][::sp_rate]
            times_used = times[index1]
            xticks = (times_used - times[0]) / np.timedelta64(1, 'h')
            xtick_labels = get_time_str_list(times_used,row=row)
            #print(xticks)
        else:
            #无规律，需穷举
            xtick_labels = get_time_str_list(times,row=row)
            xticks = []
            for i in range(len(xtick_labels)):
                xticks.append((times[i] - times[0]) / np.timedelta64(1, 'h'))
            xticks = np.array(xticks)
    else:
        pass
    return xticks, xtick_labels


def get_y_ticks(ticks,height,fonsize):
    w_one_tick = 0.3
    max_tick_num = int(height / w_one_tick)
    tick0 = ticks[0]
    if isinstance(tick0,datetime.datetime) or isinstance(tick0,np.datetime64):
        ticks1 = []
        if  isinstance(tick0,datetime.datetime):
            for tick in ticks:
                ticks1.append(meteva.base.time_tools.all_type_time_to_time64(tick))
        else:
            ticks1 = ticks
        ticks1 = list(set(ticks1))
        ticks1.sort()
        times = np.array(ticks1)
        dtimes = (times[1:] - times[0:-1])
        dhs =  dtimes / np.timedelta64(1, 'h')
        dhs_set = set(dhs.tolist())
        dh_max = (times[-1] - times[0]) / np.timedelta64(1, 'h')
        # 判断是否为有规律的
        if(dhs.size / len(dhs_set) >3):
            #有规律
            dhs_units = np.array(list(dhs_set))
            dhs_units.sort()
            dhs_u0 = dhs_units[0]
            nt = dh_max/dhs_u0
            sp_rate = int(math.ceil(nt/max_tick_num))
            dhs_u1 = dhs_u0 * sp_rate
            if dhs_u1 ==1:
                hour_list = np.arange(24).tolist()
            elif dhs_u1 <=3:
                hour_list = np.arange(2,24,3).tolist()
            elif dhs_u1 <=6:
                hour_list = np.arange(2,24,6).tolist()
            elif dhs_u1 <=12:
                hour_list = np.arange(8,24,12).tolist()
            elif dhs_u1 <=24:
                hour_list = [8]
            else:
                hour_list = [8]
            pd_times = pd.Series(0, index=times)
            index1 = np.where(pd_times.index.hour.isin(hour_list))
            times_used = times[index1]
            yticks = (times_used - times[0]) / np.timedelta64(1, 'h') / dhs_u0
            ytick_labels = get_time_str_list(times_used,row=1)

        else:
            #无规律，需穷举
            ytick_labels = get_time_str_list(times,row=2)
            yticks = np.arange(ytick_labels)
    else:
        sp =int(math.ceil( fonsize * 0.015 * len(ticks) / height))
        yticks = np.arange(0,len(ticks),sp)
        ytick_labels = ticks[::sp]
    return yticks, ytick_labels



import math

import numpy as np
import pandas as pd

#from .kernel2dmeitsjer import kernel2dmeitsjer
#from .zapsmall import zapsmall

from  meteva.method.space.pphindcast.kernel2dmeitsjer import kernel2dmeitsjer
from meteva.method.space.pphindcast.zapsmall import zapsmall


def kernel2dsmooth(x, kernel_type=None, k=None, w=None, x_object=None, xdim=None, nxy=None, setup=False, verbose=False,
                   **args):
    if xdim is None:
        xdim = list(x.shape)
    if nxy is None:
        nxy = np.prod(xdim[1: 2])
    if w is None:
        if kernel_type is not None:
            k = kernel2dmeitsjer(func_type=kernel_type, **args)
        elif k is None:
            print("kernel2dsmooth: must give a value for at least one of kernel.type, K, or W")
            return
        kdim = list(k.shape)
        bigdim = np.subtract(np.sum([xdim, kdim], axis=0), [1, 1])
        if bigdim[0] <= 1024:
            bigdim[0] = 2 ** np.ceil(math.log2(bigdim[0]))
        else:
            bigdim[0] = np.ceil(bigdim[1] / 512) * 512
        if bigdim[1] <= 1024:
            bigdim[1] = 2 ** np.ceil(math.log2(bigdim[1]))
        else:
            bigdim[1] = np.ceil(bigdim[1] / 512) * 512
        kbig = np.zeros((bigdim[0], bigdim[1]))
        kcen = list(map(math.floor, (np.array(kdim) + 1) / 2))
        kbig[0: (kdim[0] - kcen[0] + 1), 0: (kdim[1] - kcen[1] + 1)] = k[kcen[0] - 1: kdim[0], kcen[1] - 1: kdim[1]]
        if kdim[0] > 1:
            kbig[(bigdim[0] - kcen[0] + 1): bigdim[0], 0: (kdim[1] - kcen[1] + 1)] = k[0: (kcen[0] - 1),
                                                                                     kcen[1] - 1: kdim[1]]
        if kdim[1] > 1:
            kbig[0: (kdim[0] - kcen[0] + 1), (bigdim[1] - kcen[1] + 1): bigdim[1]] = k[kcen[0] - 1: kdim[0],
                                                                                     0: (kcen[1] - 1)]
        if kdim[0] > 1 and kdim[1] > 1:
            kbig[(bigdim[0] - kcen[0] + 1): bigdim[0], (bigdim[1] - kcen[1] + 1): bigdim[1]] = k[0: (kcen[0] - 1),
                                                                                               0: (kcen[1] - 1)]
        if verbose:
            print("Finding the FFT of the kernel matrix.\n")
        w = np.fft.fft2(kbig) / np.prod(bigdim)
        if verbose:
            print("FFT of kernel matrix found.\n")
        if setup:
            return w
    else:
        bigdim = w.shape
    out = np.zeros((bigdim[0], bigdim[1]))
    if type(x) == pd.DataFrame:
        x = x.fillna(0)
    out[0: xdim[0], 0: xdim[1]] = x
    # out[pd.DataFrame(out).isna] = 0
    if verbose:
        print("Performing the convolution.\n")
    if x_object is not None:
        out = (np.fft.ifft2(x_object * w)).real[0: xdim[0], 0: xdim[1]]
    else:
        out = ((np.fft.ifft2(np.fft.fft2(out) * w)) * out.size).real[0: xdim[0], 0: xdim[1]]
    if verbose:
        print("The convolution has been carried out.\n")
    return zapsmall(out)
